<?php

/**
  * OntoWiki General Action Controller Class
  * 
  * Extends the Zend_Controller_Action class and adds
  * further functionalities.
  * 
  * @author AKSW Team
  * @version $Id$
  */
abstract class OntoWiki_Controller_Action extends Zend_Controller_Action {
	
	/**
	 * @var instance of log object
	 */
	protected $log = null;
	
	/**
	 * @var instance of config object
	 */
	protected $config = null;
	
	/**
	  * @var base path for the currently selected theme
	  */
	protected $themeBase;

	/**
	 * @var object erfurt default store api
	 */
	protected $erfurtStore = null;
	
	/**
	 * @var object erfurt-app
	 */
	protected $erfurtApp = null;

	/**
	 * @var the currently chosen model
	 */
	protected $model = null;
	
	/**
	 * @var the current instance
	 */	
	protected $resource = null;
	
	/**
	 * @var the current class
	 */
	protected $class = null;
	
	/**
	 * @var Convinience variable that holds the current Zend_Session object.
	 */
	protected $session = null;
	
	/**
	 * @var instance of access control object
	 */
	protected $ac = null;
	
	/**
	  * @var Array for registering onLoad scripts
	  */
	protected $onLoadRegistry = array();
	
	/**
	  *  
	  * @var Object event handler
	  */
	protected $ed = null;

	/**
	  *  
	  * @var Object event handler
	  */
	public $triggerHelper = null;

	/**
	  * Constructor
	  */
	public function init() {
	
		// init controller variables
		$this->config      = Zend_Registry::get('config');
		$this->strings     = Zend_Registry::get('strings');
		$this->erfurtApp   = Zend_Registry::get('erfurt');
		$this->ed          = Zend_Registry::get('erfurt')->getEventDispatcher();
		$this->erfurtStore = Zend_Registry::get('erfurt')->getStore();
		$this->ac          = Zend_Registry::get('ac');
		$this->log         = Zend_Registry::get('owLog');
		$this->themeBase   = $this->config->themeBase;
		
		// save current action controller to Zend Registry
		Zend_Registry::set('runningActionController', $this);

		try {
			$this->session = Zend_Registry::get('session');
			$this->model   = $this->config->activeModel;
		} catch (Exception $e) {
			$this->log->debug('Uncatched Exception: ' . __CLASS__ . ':' .
			                  $this->message . PHP_EOL . 
			                  $this->file . ':' . 
			                  $this->line);
		}
		
		// write action object to view
		$this->view->setAction($this);
		
		// set view variables also needed by subclassed controllers
		$this->view->inlineEditingEnabled = $this->session->inlineEditingEnabled;
		$this->view->themeBase  = $this->themeBase;
		$this->view->charset    = $this->config->encoding;
		$this->view->showNews   = $this->config->news ? true : false;
		$this->view->nameSpaces = array( // TODO: generalise
			array('prefix' => 'owl',    'uri' => 'http://www.w3.org/2002/07/owl/'), 
			array('prefix' => 'SysOnt', 'uri' => 'http://ns.ontowiki.net/SysOnt/')
		);
		
		// disable layout for Ajax requests
		if ($this->_request->isXmlHttpRequest()) {
			$this->_helper->layout()->disableLayout();
		}
		
		// assign strings to view
		$this->view->strings = $this->strings;
		
		// set paths needed in JavaScript
		$this->view->erfurtUriBase   = $this->config->uriBase . $this->config->erfurtLib;
		$this->view->erfurtPublicUri = $this->config->erfurtPublicUri;
		$this->view->owUriBase       = $this->config->uriBase;
		
		// initialise response headers
		$this->initResponseHeaders();
		
		// set new language before fetching controllers
		if (isset($this->_request->lang)) {
			$this->session->modelLanguage = $this->_request->lang;
		}
		
		// set default sidebar
		$this->view->placeholder('sidebar')->set($this->view->partial('partials/sidebar.php'));
		// Zend_Registry::get('owLog')->debug('.');
		
		if (!isset($this->session->count)) {
			$this->session->count = 15;
		}
	
		// get helper to trigger Zend event methods
		$this->triggerHelper = $this->_helper->getHelper('TriggerZendEventMethods');
	}
	
	/**
	  * Post dispatcher method called after each action.
	  */
	public function postDispatch () {
		
		$session = Zend_Registry::get('session');
		
		// set some switches and vars for the view
		$routing = $this->_request->getModuleName() . '_' . $this->_request->getControllerName() . '_' . $this->_request->getActionName();
		$this->view->setOn($routing);
		$this->view->setOn(substr($routing, strlen($this->_request->getModuleName() . '_'))); // short version without default_ (module name)
		$this->view->addVar('routing', $routing);
		
		// build title
		$title = '';
		if (null !== $this->resource) {
			if ($class = $this->resource->getClass()) {
				$title .= $class->getTitle($session->modelLanguage, 0, 30) . ' : ';
			}
			$title .= $this->resource->getTitle($session->modelLanguage, 0, 30) . ' — ';
		}
		// add application title from config
		$title .= $this->config->title;
		
		// store title in view helper
		$this->view->headTitle($title, 'SET');
		
		// gather widget scripts and styles
		$wf = Erfurt_Plugin_Widget_Factory::getInstance();
		$this->view->scripts        = $wf->getRequiredScripts();
		$this->view->stylesheets    = $wf->getRequiredStylesheets();
		$this->view->onLoadRegistry = $this->onLoadRegistry;
		
		// gather plugin onload actions
		foreach ($wf->getRequiredOnLoadActions() as $action) {
			$this->view->onLoadRegistry[] = $action;
		}
		
		// We have to trigger PostDispatch here because Zend is running some stuff
		// in a strange order and ZF is rendering the view before all postDispatch
		// methods has been called (plugin->postDispatch and helper->postDispatch).
		// So we cannot use a plugin or helper to alter the view object during
		// postDispatch before it's getting rendered. The only clean solution for
		// this issue would be to overwrite some methods in Zend Controller Front
		// and Zend Dispatcher. I love it!
		/*
		$this->ed->trigger('ZendPostDispatch', $this->triggerHelper);
        $this->ed->trigger(
			$this->_request->getModuleName() . '_' . 
			$this->_request->getControllerName() . '_' . 
			$this->_request->getActionName() . '_ZendPostDispatch', $this->triggerHelper
		);
		
		2008-05-15 by Haschek:
		This is now done through the TriggerZendEventMethods helper class from
		OntoWiki, but it is experimental right now. If it proves to work then we
		will remove this whole commented block :)
        */

		// delete current action controller from Zend Registry
		Zend_Registry::set('runningActionController', null);

	}
	
	/**
	 * SuperClassPath
	 */
	function getSuperClassPath() {
		$return = false;
		
		$session = Zend_Registry::get('session');
		
		if ($this->resource) {
			$class = $this->resource->getClass();
			
			if ($class instanceof RDFSClass) {
				foreach ($class->getSuperClassPath() as $cl) {
					
					// exclude owl:Thing from classpath as it is obvious
					if ($cl->getQualifiedName() != 'owl:Thing') {
						$return .= '<a about="' . $cl->getUri() . '" href="' . OntoWiki_Util::getURL($cl, null, 'list') . '">' . 
								   $cl->getTitle($session->modelLanguage) . '</a>&nbsp;&gt; ';
					}
				}
			}
			
			$return .= $this->resource->getTitle($session->modelLanguage);
		}
		
		return $return;
	}
	
	/**
	 * Draws the tabs and their border
	 *
	 * @param $tabs
	 * @param $current
	 * @TODO: move border css to style sheet
	 */
	function getTabs(array $tabs, $current = 0) {
		$ret = '';
		
		$a = array('tabs'=>$tabs, 'currentElement'=>$current);
		
		$this->ed->trigger('template_window_create_tabs', $a);
		
		//list ($tabs, $current) = $a;
		
		$tabs = $a['tabs'];
				
		foreach ($tabs as $url => $label) {
			$ret .= '    <li' . ($current == $i++ ? ' class="active"' : '') . '><a href="' . $url . '">' . $label . '</a></li>' . PHP_EOL;
		}
		
		return $ret;
	}
	
	/**
	  * Default action called if no suitable action method found.
	  */
	public function __call($method, $args) {
		$controller = $this->_request->getControllerName();
		$action     = $this->_request->getActionName();
		
		$this->_helper->viewRenderer->setNoRender(true);
		
		echo 'Not found: ' . $controller . '::' . $action;
	}

    /**
      * returns a url without zf components
      * 
      * FIX: VERY HOT HACK
      * FIX: sorting right?
      */
    function getOwUrl() {
    	$ret = '';
    	
    	foreach ($this->_getAllParams() as $part => $stuff) {
    		if (!in_array($part, array('controller', 'module', 'action'))) {
    			$ret .= ($ret != '') ? '/' : '';
    			$ret .= $part;
    			$ret .= (strpos($part, ':') !== false) ? '/' : '';
    			if ($stuff == '') continue;
    			$ret .= '/';
    			$ret .= $stuff;
    		}
    	}
    	
    	return $ret;
	}
	
	/**
	  * Initialises the response headers with content type and char set
	  */
	protected function initResponseHeaders() {
        $charSet = $this->config->encoding;
        $this->getResponse()->setHeader('Content-Type', 'text/html; charset=' . $charSet);
    }
	
	/**
	  *	Returns the Recents box as a string.
	  *
	  * @return string
	  */ 
	function getRecentBox() {
		global $model;
		$changes_content = '<div class="changes_content" name="recents">';
		if ($this->model) {
			foreach(OntoWiki_Util::listComments($instance, '', 0, 5, &$erg) as $comment) {
				$changes_content .= '<p>' . $comment->added . '&nbsp;' . 
						$comment->user . ':<br />&nbsp;&nbsp;&nbsp;<a href="' . 
						OntoWiki_Util::getURL($comment->subj) . '">' . 
						OntoWiki_Util::getTitle($comment->subj, true, 20) . '</a>->' . 
						OntoWiki_Util::getTitle($comment->pred, true, 20) . '->' . 
						OntoWiki_Util::getTitle($comment->obj, true, 20) . '<br />' . 
						$comment->reified->obj->getLabel() . '</p>';
			}
			if ($erg > 5) {
				$changes_content .= '<a href="' . 
						($instance ? OntoWiki_Util::getURL($instance) : 'discussion.php') . 
						'">Show&nbsp;all&nbsp;' . $erg . '</a>&nbsp;&gt;&gt;</a>';
			}
		}
		$changes_content .= '</div>' . PHP_EOL . 
		                    '<div class="changes_content" name="recents">Comments</div>' . PHP_EOL;
		
		$changes_box = new OntoWiki_Html_LayoutBox(array(
					'<a href="javascript:ow.optionalToggle(\'recents\',0)" name="recents">' . $this->strings->default->changes->title . '</a>', 
					'<a href="javascript:ow.optionalToggle(\'recents\',1)" name="recents">' . $this->strings->default->comments->title . '</a>'),
				'changes', 
				'', 
				!($this->session->changes_box == 'closed'));
		
		$changes_box->addString($changes_content);
		
		return $changes_box->__toString();
	}
	
	/**
	  *	Returns the current model
	  *
	  * @return null|object
	  */ 
    public function getModel() {
        return $this->model;
    }

	/**
	  *	Returns eventdispatcher object
	  *
	  * @return null|object
	  */ 
    public function getEventDispatcher() {
        return $this->ed;
    }

	/**
	  *	only render existing views (or take the default view)
	  *
	  * @return false or nothing
	  */ 
    public function renderView($name) {
        $paths = $this->view->getScriptPaths();
        $suffix = '.'.$this->_helper->viewRenderer->getViewSuffix();
        
        $renderit = false;
        
        if ($paths && is_array($paths) && count($paths)>0) {
            foreach ($paths as $path) {
                if (is_readable($path.$name.$suffix)) {
					$renderit = true;
				}
            }

            if ($renderit === true) {
                $this->renderScript($name.$suffix);
                $this->_helper->viewRenderer->setRender($name);
                return;
            }
            else {
                return false;
            }
        }
        else {
            return false;
        }
    }
}

?>
