<?php

/**
  * Trigger Zend event methods automatically
  *
  * @author Michael Haschke
  * @version $Id$
  * @package OntoWiki
  * @subpackage	Controller
  */
class OntoWiki_Controller_Plugin_TriggerZendEventMethods extends Zend_Controller_Plugin_Abstract {
	
    private $_ed = null;
    public $request = null;

    function __construct(Erfurt_EventDispatcher $ed) {
        $this->_ed = $ed;
    }

    public function routeStartup(Zend_Controller_Request_Abstract $request)
    {
        $this->request = $request;
        $this->_ed->trigger('ZendRouteStartup',$this);
        # only one trigger because exact rout is not know at this point
    }

    public function routeShutdown(Zend_Controller_Request_Abstract $request)
    {
        $this->request = $request;
        $this->_ed->trigger('ZendRouteShutdown',$this);
        $this->_ed->trigger($request->getModuleName().'_'.$request->getControllerName().'_'.$request->getActionName().'_ZendRouteShutdown',$this);
    }

    public function dispatchLoopStartup(Zend_Controller_Request_Abstract $request)
    {
        $this->request = $request;
        $this->_ed->trigger('ZendDispatchLoopStartup',$this);
        $this->_ed->trigger($request->getModuleName().'_'.$request->getControllerName().'_'.$request->getActionName().'_ZendDispatchLoopStartup',$this);
    }

    /* 
     * Temporarily renamed these triggers with *_deprecated to make them work.
     * General solution for late/early call (Pre-Dispatch/Template-triggers) of _init in
     * Plugins is needed if it is called early no _view var is available in plugins
     * if called late no real preDispatch actions are possible that means controller
     * has eventually already been dispatched to e.g. error-controller
     * (Christop Rieß)
     */
    public function preDispatch(Zend_Controller_Request_Abstract $request)
    {
        $this->request = $request;
        $this->_ed->trigger('ZendPreDispatch_deprecated',$this); // not needed anymore but here as long as some plugins use it
        $this->_ed->trigger('ZendPreDispatch',$this);
        $this->_ed->trigger($request->getModuleName().'_'.$request->getControllerName().'_'.$request->getActionName().'_ZendPreDispatch',$this);
    }

    /*
    public function postDispatch(Zend_Controller_Request_Abstract $request)
    {
        done by controller action helper
    
        $this->request = $request;
        $this->_ed->trigger('ZendPostDispatch_deprecated',$this);
        $this->_ed->trigger($request->getModuleName().'_'.$request->getControllerName().'_'.$request->getActionName().'_ZendPostDispatch',$this);
    }
    */

    public function dispatchLoopShutdown()
    {
        $this->_ed->trigger('ZendDispatchLoopShutdown',$this);
    }

}

?>
