<?php

class OntoWiki_Helper_MakeList {
	
	/**
	  * Creates an HTML list out of an array passed.
	  * The last list item will automatically get the css class "last-child".
	  *
	  * @param array An array of list items.
	  * @param string $type The list type ('ul'|'ol').
	  * @param string A string of css classes for the list tag.
	  * @todo  Support for definition lists (&lt;dl&gt;) and support for list item classes.
	  *
	  * @return string A string of HTML with the list.
	  */ 
	public function makeList(array $items, $type = 'ul', $cssClasses = null, $itemClasses = null) {
		$ret = '<' . $type;
		if (is_string($cssClasses)) {
			$ret .= ' class="' . $cssClasses . '"';
		}
		$ret .= '>' . PHP_EOL;
		
		$i = 0;
		$count = count($items);
		foreach ($items as $item) {
			if (++$i != $count) {
				$ret .= '<li' . ($itemClasses ? (' class="' . $itemClasses . '"') : '') . '>' . $item . '</li>' . PHP_EOL;
			} else {
				$ret .= '<li class="last-child' . ($itemClasses ? (' ' . $itemClasses . '"') : '"') . '>' . $item . '</li>' . PHP_EOL;
			}
		}
		
		$ret .= '</' . $type . '>' . PHP_EOL;
        return $ret;
	}
}

?>