<?php
/**
 * OntoWiki Plugin
 *
 * Base class for OntoWiki plugins:
 * class YourOntoWikiPluginClass extends OntoWiki_Plugin {}
 *
 * @package plugin
 * @package ontowiki
 * @author  Michael Haschke
 * @version $Id:$
 */
class OntoWiki_Plugin extends Erfurt_Plugin {

    protected $_action = null;
    protected $_view = null;
    protected $_viewRenderer = null;

    protected function _initPlugin($erfurtObject) {
		parent::_initPlugin($erfurtObject);
		
		// check for strings.ini file in plugin dir and merge with ow strings config
		$pluginRoot = $this->_getPluginRootDir();
		$strFileName = $pluginRoot . '/strings.ini';
		
		if (file_exists($strFileName) && is_readable($strFileName)) {
		    $config = Zend_Registry::get('config');
		    $strConfig = new Zend_Config_Ini($strFileName, (isset($config->language->gui)) ? $config->language->gui : 'default',
		            true);
		            
		    $mergedConfig = Zend_Registry::get('strings')->merge($strConfig);
		    Zend_Registry::set('strings', $mergedConfig); 
		}
		
		// set environment when plugin is requested the first time
		$this->_setEnvironment();
		
        return;
	}
	
	/**
	  * set environment for plugins after ZendPostDispatch,
	  * if your plugin is instanciated before dispatching then you need to call
	  * this method manually
      */
	protected function _setEnvironment() {
	
		// get active action controller object
		if (Zend_Registry::isRegistered('runningActionController')
		    && Zend_Registry::get('runningActionController') != null) {
			$this->_action = Zend_Registry::get('runningActionController');
		}
		else {
		    $this->_action = null;
		}
		
	    // get viewRenderer helper from Helper Broker when action object does not exist
	    // or an existing action object using the viewRenderer
	    if (Zend_Controller_Action_HelperBroker::hasHelper('viewRenderer') &&
	        ($this->_action === null || ($this->_action != null && !$this->_action->getInvokeArg('noViewRenderer')))) {
	        $this->_viewRenderer = Zend_Controller_Action_HelperBroker::getExistingHelper('viewRenderer');
	    } else {
	        $this->_viewRenderer = null;
	    }

	    // get view object from active action controller or viewRenderer helper
	    if (isset($this->_action->view)) {
		    $this->_view = $this->_action->view;
		} elseif ($this->_viewRenderer !== null) {
		    $this->_view = $this->_viewRenderer->view;
	    } else {
	        $this->_view = null;
	    }
				
		/*
		// add plug-in basedir as script path
		if ($this->_view) {
			$this->_view->addScriptPath($this->_getPluginRootDir());
		}
		commented out by Haschek (2008-05-15): this is nonsense, should be made by plugin author when it's needed
		*/
		
        return;
	}
	
	/**
	  *
	  * @return plugin's web base dir or false
      */
	protected function _getPluginBaseUri() {
		if ($config = Zend_Registry::get('config')) {
			return rtrim(str_replace(REAL_BASE,$config->uriBase,$this->_getPluginRootDir()),DIRECTORY_SEPARATOR);
		} else {
		    return false;
		}
	}
	
	/**
	  *
	  * @return OntoWiki app's web base dir or false
      */
	protected function _getOntoWikiBaseUri() {
		if ($config = Zend_Registry::get('config')) {
			return rtrim($config->uriBase,DIRECTORY_SEPARATOR);
		} else {
		    return false;
		}
	}
	
    // get active model
	protected function _getActiveModel() {
        $config = Zend_Registry::get('config');
        if ($config->activeModel) {
            return $config->activeModel;
        } else {
            return false;
        }
	}

}

?>
