<?php

/**
  * OntoWiki view class.
  *
  * @author Norman Heino
  * @version $Id: View.php 1841 2008-02-03 20:44:40Z haschek $
  * @license GPL
  */
class OntoWiki_View extends Zend_View {
	
	/* --- old vars --- */
	
	/**
	  * @var Holds the sub view for this view instance.
	  *      Each OntoWiki_View has exactly one or no direct sub view
	  *      which in turn can have a sub view.
	  */
	protected $subView = null;
	
	/* --- new vars --- */
	
	/** @var OntoWiki Action object
	  */
	protected $owAction = null;
	
	protected $boolVars = array();
	
	protected $infoVars = array();
	
	/* --- old methods --- */
	
	/** saves the current action object to view
	  */
	public function setAction($action)
	{
	    $this->owAction = $action;
	}
	
	/**
	  * Returns an OntoWiki_View object that can be used to render subtemplates.
	  *
	  * @param boolean $assignVars All variables assigned to this OntoWiki_View object will be
	  *        assigned to the subView returned.
	  * @return OntoWiki_View
	  */
	public function getSubView($assignVars = false) {
		if (null === $this->subView) {
			$this->subView = new OntoWiki_View(array(
				'scriptPath' => $this->getScriptPaths(), 
				'encoding'   => $this->getEncoding())
			);
		}
		
		// clear variables from preceding calls
		$this->subView->clearVars();
		
		if ($assignVars) {
			$vars = $this->getVars();
			
			if (is_array($vars)) {
				$this->subView->assign($vars);
			}
		}
		
		return $this->subView;
	}
	
	/**
	  * Creates an HTML list out of an array passed.
	  * The last list item will automatically get the css class "last-child".
	  *
	  * @param array An array of list items.
	  * @param string $type The list type ('ul'|'ol').
	  * @param string A string of css classes for the list tag.
	  * @todo  Support for definition lists (&lt;dl&gt;) and support for list item classes.
	  *
	  * @return string A string of HTML with the list.
	  */ 
	// public function makeHtmlList(array $items, $type = 'ul', $cssClasses = null, $itemClasses = null) {
	// 	$ret = '<' . $type;
	// 	if (is_string($cssClasses)) {
	// 		$ret .= ' class="' . $cssClasses . '"';
	// 	}
	// 	$ret .= '>' . PHP_EOL;
	// 	
	// 	$i = 0;
	// 	$count = count($items);
	// 	foreach ($items as $item) {
	// 		if (++$i != $count) {
	// 			$ret .= '<li' . ($itemClasses ? (' class="' . $itemClasses . '"') : '') . '>' . $item . '</li>' . PHP_EOL;
	// 		} else {
	// 			$ret .= '<li class="last-child' . ($itemClasses ? (' ' . $itemClasses . '"') : '"') . '>' . $item . '</li>' . PHP_EOL;
	// 		}
	// 	}
	// 	
	// 	$ret .= '</' . $type . '>' . PHP_EOL;
	// 	return $ret;
	// }
	
	/* =========================================================================
	   == new functions for replacement holders in theme =======================
	   == @autor: Michael Haschke ==============================================
	   ====================================================================== */

	/** used in template to trigger an action which can used by
	  * plugins to add their stuff on that place
	  */
	public function process($place) {
	    $ed = $this->owAction->getEventDispatcher();
	    $processStr = null;
	    $ed->trigger('template_'.$place,&$processStr);
	    return $processStr;
	}
	
	/** returns a string with all used namespaces related to a prefix
	  * o usage suggested in body element
	  */
	public function xmlnsPrefixes($xmlns = array()) {
	    
	    $prefixes = array();
	    
	    if (is_array($xmlns) && count($xmlns)>0) {
	        foreach ($xmlns as $ns)
	            $prefixes[] = 'xmlns:'.$ns['prefix'].'="'.$ns['uri'].'"';
	    }
	    
	    return implode(' ',$prefixes);
	}
	
	/** create array with contents like OW App menu, search field, etc
	  */
	public function owBoxApplication() {
	    return $this->owAction->getOntoWikiBox2();
	}

	/** create array with contents for managed models
	  */
	public function owBoxModels() {
	    return $this->owAction->getModelsBox2();
	}

	/** create array with contents for different languages used in current model
	  */
	public function owBoxLanguages() {
	    return $this->owAction->getLanguageBox2();
	}

	/** create array with contents for current model's classes
	  */
	public function owBoxClasses() {
	    return $this->owAction->getClassBox2();
	}

	/** create array with contents for popular (most viewed) resources in model
	  */
	public function owBoxPopular() {
	    return $this->owAction->getPopularBox2();
	}

	/** create array with contents for OntoWiki login
	  */
	public function owBoxLogin() {
	    return $this->owAction->getLoginBox2();
	}
	
	/** check for an authentification
	  * o returns username or nothing
	  * @return string|null
	  */
	public function owCurrentUser() {
		$auth = Zend_Auth::getInstance();
		if ($auth->hasIdentity()) {
		    $usr = $auth->getIdentity();
		}
		
		if (!$auth->hasIdentity() || (isset($usr['anonymous']) && $usr['anonymous'] != 0)) {
		    # no user auth found
		    return null;
		}
		else {
		    # user auth found, return user name
		    return $usr['username'];
		}
    }
     
	/** check for selected model
	  * o returns modeluri or nothing
	  * @return string|null
	  */
	public function owCurrentModel() {
	
	    $model = $this->owAction->getModel();
	
		if (!$model) {
		    # no model found
			return null;
		}
		else
		{
		    # model found, return uri
		    return $model->modelURI;
		}
    }
    
    /** set namespace/var = true (like switch = on)
      * @return null
      */
    public function setOn($varname = null, $ns = 'default') {
        if ($varname != '' && $ns != '') {
            $this->boolVars[$ns][$varname] = true;
        }
        return null;
    }
    
    /** check namespace/var for existence (like a on|off switch)
      * @return bool
      */
    public function is($varname = null,$ns='default') {
        
        if ($varname != '' && $ns != '') {
            if (isset($this->boolVars[$ns][$varname]) && $this->boolVars[$ns][$varname] === true) {
                return true;
            }
            else {
                return false;
            }
        }
        else {
            return false; # maybe error because empty varname
        }
        
    }

    /** set namespace/var = false (like switch = off)
      * @return null
      */
    public function setOff($varname = null, $ns = 'default') {
        if ($this->is($varname,$ns)) {
            unset($this->boolVars[$ns][$varname]);
        }
        return null;
    }
    
    /** set namespace/var = value
      * @return bool|null
      */
    public function addVar($varname = null, $value, $ns = 'default') {
        if ($varname != '' && $ns != '') {
            $this->infoVars[$ns][$varname] = $value;
            if (isset($this->infoVars[$ns][$varname]) && $this->infoVars[$ns][$varname] === $value) {
                return true;
            }
            else {
                return false;
            }
            
        }
        else
        {
            return null;
        }
    }
    
    /** get value of namespace/var
      * @return mixed|false
      */
    public function get($varname = null, $ns = 'default') {
        if ($varname != '' && $ns != '' && isset($this->infoVars[$ns][$varname])) {
            return $this->infoVars[$ns][$varname];
        }
        else
        {
            return false;
        }
    }
    
    /** delete namespace/var
      * @return bool
      */
    public function delVar($varname = null, $ns = 'default') {
        if ($varname != '' && $ns != '' && isset($this->infoVars[$ns][$varname])) {
            unset($this->infoVars[$ns][$varname]);
            if (isset($this->infoVars[$ns][$varname])) {
                return false;
            }
            else {
                return true;
            }
        }
        else
        {
            return true;
        }
    }



}

?>
