<?php
/**
 * ContainerManager plugin controller
 *
 * @author Maria Moritz
 * @version $Id$
 */

class ServiceController extends OntoWiki_Controller_Action {
	private $container;
	
   /**
     * all ContainerManager actions will be processed here
     */
	public function containerAction() {

		// gets the strings from the strings.ini
		$this->_strings = Zend_Registry::get('strings');
		
		// Disable layout (we want only one or more boxes no full ontowiki layout)
		$this->_helper->layout->disableLayout();
		
		// Add the path of templates needed by this plugin
		$this->view->addScriptPath(REAL_BASE.'plugins/ContainerManager/templates/service');
		
		// Needed here because of the automatic rendering should be deactivated
		$this->_helper->viewRenderer->setNoRender(true);
		
		// takes the front controller for the request objekt
        $frontController = Zend_Controller_Front::getInstance();
		$request = $frontController->getRequest();
		
		// gets the active model	
		$this->model = Zend_Registry::get('config')->activeModel;
		$activeModelU = $this->model->modelURI;
		// get all request parameters
		$action = $request->act;
		$name = $request->name;
		$r = $request->r;
		$title = $request->t;
		$entry = $request->entry;
		$modelU = $request->m;
		$page = $request->p;
		$bool = false;
		$modelUri = strtolower(isset($modelU)? $modelU : $activeModelU );
		
		//if (!isset($_SESSION['container'])) {
		if (!isset($this->container)) {
			if($_SESSION['container']) {
				$this->container = $_SESSION['container'];
				$_SESSION['message'] = null; 
			}
			else {
				$this->container = array();
				$_SESSION['container'] = $this->container;
				$_SESSION['message'] = null;	
			}
		}
		
		if ($name != null ) {
			
			$this->container = $_SESSION['container'];
			switch($action){
				// adds a container to the array $container
				case 'add' : {
					if(array_key_exists($name, $this->container)) {
					
						//$_SESSION['message'] = '<p class="message error">' . $this->_strings->cm->error->container . '</p>';
						$this->view->message = '<p class="message error">' . $this->_strings->cm->error->container . '</p>';
					
						//echo $this -> view -> render('message.php');
						echo $this -> view -> render('box.php');
						//header('HTTP/1.0 400 Bad Request');
						exit;
					}
					$this->container[$name] = array();
					ksort($this->container);
					$_SESSION['container'] = $this->container;
					echo $this -> view -> render('box.php');					
					
				} break;
				
				// adds a resource if entered manuelly 
				case 'addr' : {
					
					$this->add($modelUri, $r, $name, $this->container, $title);
					$this->pager($name, $page);
					echo $this -> view -> render('list.php');
														
				} break;
				
				// adds a resource if entered via drag & drop
				case 'addrN' : {
					$this->add($modelUri, $r, $name, $this->container, $title);
					echo $this -> view -> render('box.php');
					
				} break;
				
				// deletes a container
				case 'del' : {
					unset($this->container[$name]);
					$_SESSION['container'] = $this->container;
					if ($_SESSION['list'][$name])  unset($_SESSION['list'][$name]); 
					echo $this -> view -> render('box.php');
					
				} break;
				
				// deletes a resource with given uri's (not in use at this time)
				case 'delr' : {
					
					if($modelUri == null || $r == null ){
						//$_SESSION['message'] = '<p class="message error">' . $this->_strings->cm->error->mod_res . '</p>';
						$this->view->message = '<p class="message error">' . $this->_strings->cm->error->mod_res . '</p>';
						//header('HTTP/1.0 500 Internal Server Error');
						//echo $this -> view -> render('message.php');
						$this->pager($name, $page);
						echo $this -> view -> render('list.php');
						exit();
					}
					for($i=0;$i<sizeof($this->container[$name]);$i++){
						if($this->container[$name][$i][0]==$r && $this->container[$name][$i][1]==$modelUri){
							unset($this->container[$name][$i]);
							break;
						}
					}
					$_SESSION['container'] = $this->container;
					echo $this -> view -> render('box.php');
										
				} break;
				
				// deletes a resource via the entry id 
				case 'delrN' : {
					
					if($entry == null ){
						//$_SESSION['message'] = '<p class="message error">' . $this->_strings->cm->error->mod_res . '</p>';
						$this->view->message = '<p class="message error">' . $this->_strings->cm->error->mod_res . '</p>';
						//header('HTTP/1.0 500 Internal Server Error');
						//echo $this -> view -> render('message.php');
						$this->pager($name, $page);
						echo $this -> view -> render('list.php');
						exit();
					}
					unset($this->container[$name][$entry]);
					$_SESSION['container'] = $this->container;
					$_SESSION['list'][$name] = $this->container[$name];
					
					$this->pager($name, $page);
					echo $this -> view -> render('list.php');
										
				} break;
				
				// deletes the resources of a container from the original model
				case 'delResources' : {
					try{
						foreach ($this->container[$name] as $key => $value){
							if ($value[1] == $activeModelU) {
								$resource = $this->model->instanceF($value[0]);	
							}
							else {
								$store = Zend_Registry::get('erfurt')->getStore();
								$model = $store->getModel($value[1]);
								$resource = $model->instanceF($value[0]);
							}
						}
						$resource->remove();
						$this->view->message = '<p class="messagebox info">'.$this->_strings->cm->box->deleted.'</p>';
					
						unset($this->container[$name]);
						$_SESSION['container'] = $this->container;
						echo $this -> view -> render('box.php');
						
					}
					catch (Exception $e){
						
						$this->view->message = '<p class="message error">' . $this->_strings->cm->error->instanceF . '</p>' . 
												'<p class="message error">' . $e->getMessage() . '</p>'	;
						echo $this -> view -> render('box.php');
					}
				} break;
				
				// empties a container
				case 'empty' : {
					$this->container[$name] = array();
					$_SESSION['container'] = $this->container;
					if ($_SESSION['list'][$name])  $_SESSION['list'][$name] = array();
					echo $this -> view -> render('box.php');
					
				} break;
				
				// exports the content of a container
				case 'export' :{
					
					$store = Zend_Registry::get('erfurt')->getStore();
					$exportModel = new MemModel();
					if ( !$this->container[$name] ) {
						header('HTTP/1.0 400 Bad Request');
						die();
					}

					/*$modelList = $this->modelSort($container[$name]);
					foreach($modelList as $key => $value) {
						$modelURI = $key;
						$resourceURI = $value[0];
						$resourceModel = $store->getModel($modelURI);
						$resource = $resourceModel->resourceF( $resourceURI );
						$exportModel->addModel( $resource->getDefiningModel() );
					}
					*/
					foreach($this->container[$name] as $key => $value){
						$modelURI = $value[1];
						$resourceURI = $value[0];
						//echo $resourceURI;
						//echo $modelURI;
						// Todo: sort for models and get one model only once
						$resourceModel = $store->getModel($modelURI);
						//if(!$resourceModel) echo "test";
						if ( $resourceModel ) {
							$resource = $resourceModel->resourceF( $resourceURI );
							if ( $resource ) {
								$exportModel->addModel( $resource->getDefiningModel() );
								
								$xmlWriter = new Erfurt_Syntax_StringWriterXMLImpl();
								$rdfSerializer = new Erfurt_Syntax_RDFSerializer();
								header('Content-Type: application/rdf+xml');
								echo $rdfSerializer->serializeToString($xmlWriter, $exportModel);
									
							}
							else {
								echo $this->view->message = '<p class="message error">'.$this->_strings->cm->box->noResourceInstance.'</p>';
							}
						}
						else {
							echo $this->view->message = '<p class="message error">'.$this->_strings->cm->box->noModelInstance.'</p>';
						}	
					}
										
					?><pre><?php
					//print_r($modelList);
					?></pre><?php
					
				} break;
				
				// lists the content of a container
				case 'list' : {
					
					// prepares the specified container in a separate array
					$_SESSION['list'] = array();
					$_SESSION['list'][$name] = $this->container[$name];
										
					if($_SESSION['list'][$name] == null && ! array_key_exists( $name , $this->container ) ){
					
						//$_SESSION['message'] = '<p class="messagebox">' . $this->_strings->cm->error->entries . '</p>';
						$this->view->message = '<p class="message error">' . $this->_strings->cm->error->entries . '</p>';
						//header('HTTP/1.0 400 Bad Request');
						//echo $this -> view -> render('message.php');
						$this->pager($name, $page);	
						echo $this -> view -> render('list.php');
						exit();
					}
					
					$this->pager($name, $page);				
					echo $this -> view -> render('list.php');
					
				} break;
				
				// other actions aren't specified
				default : {
					//$_SESSION['message'] = '<p class="message error">' . $this->_strings->cm->error->entries . '</p>';
					$this->view->message = '<p class="message error">' . $this->_strings->cm->error->entries . '</p>';
					//header('HTTP/1.0 400 Bad Request');
					//echo $this -> view -> render('message.php');
					echo $this -> view -> render('box.php');
					exit;
				}
			}
		}
		// display just the container box if those parameters aren't set
		else if ($name == null && $action == null && $r == null ){
			/*
			?><pre><?php
			print_r($_SESSION['container']);
			?></pre><?php
			*/	
			echo $this -> view -> render('box.php');
		}
		// check the entries
		else {
			//$_SESSION['message'] = '<p class="messagebox">' . $this->_strings->cm->error->entries . '</p>';
			$this->view->message = '<p class="messagebox">' . $this->_strings->cm->error->entries . '</p>';
			//header('HTTP/1.0 400 Bad Request');
			//echo $this -> view -> render('message.php');
			echo $this -> view -> render('box.php');
		}	
	}
	
	// generalizes the add actions
	function add($modelUri, $r, $name, $container, $title){
					
		if ($modelUri == null || $r == null || $modelUri == 'undefined'|| $r == 'undefined' || $title == 'undefined' || $title == null){
			//$_SESSION['message'] = '<p class="massage error">' . $this->_strings->cm->error->mod_res . '</p>';
			$this->view->message = '<p class="massage error">' . $this->_strings->cm->error->mod_res . '</p>';
			//header('HTTP/1.0 500 Internal Server Error');
			//echo $this -> view -> render('message.php');
			return;
		}
		if (!array_key_exists($name, $container)) {
			//$_SESSION['message'] = '<p class="message error">' . $this->_strings->cm->error->nContainer . '</p>';
			$this->view->message = '<p class="message error">' . $this->_strings->cm->error->nContainer . '</p>';
			//header('HTTP/1.0 400 Bad Request');
			//echo $this -> view -> render('message.php');
			return;
		}
		for($i=0;$i<sizeof($container[$name]);$i++){
			if($container[$name][$i][0]==$r && $container[$name][$i][1]==$modelUri){
				//$_SESSION['message'] = '<p class="message error">' . $this->_strings->cm->error->entry . '</p>';
				$this->view->message = '<p class="message error">' . $this->_strings->cm->error->entry . '</p>';
				//header('HTTP/1.0 400 Bad Request');
				//echo $this -> view -> render('message.php');
			return;
			}
		}		

		$container[$name][] = array($r, $modelUri, $title);
		asort($container[$name]);
		$_SESSION['container'] = $container;
								
		$_SESSION['list'] = array();
		$_SESSION['list'][$name] = $container[$name];
									
		if($_SESSION['list'][$name] == null && !(array_search(null, $container) == $name )){
						
			//$_SESSION['message'] = '<p class="messagebox">' . $this->_strings->cm->error->entries . '</p>';
			$this->view->message = '<p class="messagebox">' . $this->_strings->cm->error->entries . '</p>';
			//header('HTTP/1.0 400 Bad Request');
			//echo $this -> view -> render('message.php');
			return;
		}		
	}
	
	// pager
	function pager($name, $page){
		// page navigation
					
		// number of entries per page
		$max = 10; 
		// number of pages
		$count_pages = ((count($_SESSION['container'][$name]))/$max);
				
		if ( isset($this->view->pagerLinks ) ) unset( $this->view->pagerLinks);
		// prepare an array for pager variables
		$this->view->pagerLinks = array();
		$this->view->pagerLinks['max'] = $max;
		$maxButtons = 10;
		$counter = 0;
					
		if (!isset($page) OR $page == 'undefined') { 
			$page = 1;
		}
		// save the page number from the request objekt to pagerLinks array
		$this->view->pagerLinks['page'] = $page;
		
		// back links for side navi
		if ($page != 1) { 
			$back = $page-1; 
			$this->view->pagerLinks['first'] = "<a class='pager' name='".$name."' id='1'>&lsaquo;&nbsp;" . $this->_strings->cm->page->first . "</a>";
			$this->view->pagerLinks['back'] = "<a class='pager' name='".$name."' id='". $back ."'>&laquo;&nbsp;" . $this->_strings->cm->page->prev . "</a>";
			$counter += 2;
		}
					
		// forward links for side navi
		if ($page < $count_pages) { 
			$forward = $page+1; 
			$this->view->pagerLinks['forward'] = "<a class='pager' name='".$name."' id='". $forward . "'>". $this->_strings->cm->page->next . "&nbsp;&raquo;</a>";
			$this->view->pagerLinks['last'] = "<a class='pager' name='".$name."' id='". $count_pages ."'>". $this->_strings->cm->page->last . "&nbsp;&rsaquo;</a>";
			$counter += 2;
		}
				
		// all other displayed pages
		if($count_pages > 1) {
			$buffer = $maxButtons - $counter;
			for ($i = 1; $i < $count_pages+1; $i++) { 
				if($i > $page-($buffer/2) && $i <= $page+($buffer/2)){
					if($i == $page) $this->view->pagerLinks['pages'][$i] = "<a class='pager selected' name='".$name."' id='". $i ."'>" . $i . "</a>";
					else {
						$this->view->pagerLinks['pages'][$i] = "<a class='pager' name='".$name."' id='". $i ."'>" . $i . "</a>";	
					}
				} 
			}
		}
	}
	
	function modelSort( $array ){
		
		$modelList = array();
		foreach($array as $key => $value){
			$modelURI = $value[1];
			$resourceURI = $value[0];
			if( array_key_exists($modelURI, $modelList)){
				array_push($modelList[$modelURI], $resourceURI);
			}
			else {
				$modelList[$modelURI] = array($resourceURI);	
			}
		}
		return $modelList;
	}
	
}
?>