<?php
/**
 * DL-Learner plugin controller
 *
 * @author Maria Moritz & Vu Duc Minh
 * @version $Id$
 */

class ServiceController extends OntoWiki_Controller_Action {
	
	private $client;
	
	// ID given to this client by the web service
	private $id;
	private $wsdluri;
	// ID of the DBpedia knowledge source
	private $ksID;
	private $OntowikiUrl;

	//init the Controller
	public function init() {
		parent::init();

		// redirect if no model has been selected
		if (!($this->model instanceof RDFSModel)) {
			$this->_redirect('');
			return;
		}
		
		if ($this->model instanceof RDFSModel) {
			try {
				if ($config->activeResource instanceof Erfurt_Rdfs_Resource ||
				isset($this->session->activeResourceUri) ) {
					$this->resource = $this->model->instanceF($this->session->activeResourceUri);
				} else if (isset($this->_request->r)) {
					$this->resource = $this->model->instanceF($this->_request->r);
				}

				// check if resource is a class
				// and forward to list view
				if ($this->resource->isClass()) {
					$this->class = $this->model->classF($this->resource);
					$this->session->activeClassUri = $this->class->getURI();
				}
				
			} catch (Exception $e) {
				$this->log->debug('Uncatched Exception: ' . __CLASS__ . ':' .$e->message . PHP_EOL . $e->file . ':' . $e->line);
			}
		} 
	}

	/**
	* connect with the DL-Learner Web Service
	*/
	function dlLearnerConnectionAction($id = 0, $ksID = 0)
	{
		ini_set('default_socket_timeout',200);
		$config = Zend_Registry::get('config');
	
		$this->OntowikiUrl= 'http://localhost/ontowiki/service/sparql';
		print_r($this->_getOntoWikiBaseUri());
		$this->client=new SoapClient("plugins/Dllearner/main.wsdl",array('features' => SOAP_SINGLE_ELEMENT_ARRAYS));//("plugins/Dllearner/main.wsdl");
		$this->id=$id;
		$this->ksID=$ksID;
	}
	
	//get the ID for the Prozess
	function getIDsAction()
	{
		$id=$this->client->generateID();
		$ksID=$this->client->addKnowledgeSource($id,"sparql",$this->OntowikiUrl);
		return array(0 => $id, 1 => $ksID);
	}
	
	/**
	* get the concepts for the learnprozess 
	*/
	function getConceptFromExamplesAction($posExamples,$negExamples)
	{
	
	    $frontController = Zend_Controller_Front::getInstance();
		$request = $frontController->getRequest();
        $options = $request->opt;
		$options = explode('/', $options);
		
		// options
		$reasoner = $options[0];
		$learningProblem = $options[1]; 
		$learningAlgorithm = $options[2];
		$this->client->applyConfigEntryInt($this->id, $this->ksID, "recursionDepth",2);
	    
		//get the negative examples when nothing there
		if(empty($negExamples)){
			$negExamples=$this->client->getNegativeExamples($this->id,$this->ksID,$posExamples,count($posExamples),"");
			$negExamples=$negExamples->item;
		}
		$this->client->applyConfigEntryStringArray($this->id, $this->ksID, "instances", array_merge($posExamples,$negExamples));
		
		$this->client->setReasoner($this->id, $reasoner);
		if(empty($negExamples))
			$this->client->setLearningProblem($this->id, "posOnlyDefinition");
		else
			$this->client->setLearningProblem($this->id, $learningProblem);
		$this->client->setPositiveExamples($this->id, $posExamples);
	
			$this->client->setNegativeExamples($this->id, $negExamples);
		$this->client->setLearningAlgorithm($this->id, $learningAlgorithm);
		
		$start = microtime(true);
		
		$this->client->initAll($this->id);

		$threaded=true;
	
	    if($threaded == false) {
	
			$concept = $this->client->learn($this->id,'manchester');
			
		} else {
		
			$this->client->learnThreaded($this->id);
			
			$i = 1;
			$sleeptime = 1;
			$learnttl=30000000;
			do {
				// sleep a while
				sleep($sleeptime);
				
				// see what we have learned so far
				
				$running=$this->client->isAlgorithmRunning($this->id);
				
				$seconds = $i * $sleeptime;
				
				$i++;
				
			
			} while($seconds<$this->ttl&&$running);
			
			$this->client->stop($this->id);
		}
		return $this->client->getCurrentlyBestEvaluatedDescriptions($this->id,4);
		
	}
	


   /**
     * all DL-Learner actions will be processed here
     */
	public function dllearnerAction() {
         
		// gets the strings from the strings.ini
		$this->_strings = Zend_Registry::get('strings');

		// Disable layout (we want only one or more boxes no full ontowiki layout)
		$this->_helper->layout->disableLayout();
		
		// Add the path of templates needed by this plugin
		$this->view->addScriptPath(REAL_BASE.'plugins/Dllearner/templates/service');
		
		// Needed here because of the automatic rendering should be deactivated
		$this->_helper->viewRenderer->setNoRender(true);
		
		// takes the front controller for the request objekt
        $frontController = Zend_Controller_Front::getInstance();
		$request = $frontController->getRequest();
		
		// gets the active model	
		$this->model = Zend_Registry::get('config')->activeModel;
		$activeModelU = $this->model->modelURI;
		$action = $request->act;
		$name = $request->name;
		$pos = $request->pos;
		$neg = $request->neg;
		$options = $request->opt;
		$id = $request->id;
		$st = $request->s;
		$cmPlugin = Zend_Registry::get('erfurt')->getPluginManager()->prepare('ContainerManager'); 
		
		// checks whether the ContainerManager Plugin is active
		if($cmPlugin == '' ){
			$_SESSION['cmPluginOff'] = '<p class="message error">' . $this->_strings->dl->message->error->cm . '</p>';
			header('HTTP/1.0 400 Bad Request');
			echo $this -> view -> render('start.php');
			return;
		}
		
		// array for active processes
		if(!isset($_SESSION['processes'])){
			$_SESSION['processes'] = array();
		}
		// array for the potentially examples for DLL-Learner
		if(!isset($_SESSION['view'])){
			$_SESSION['view'] = array();
		}
		
		if($action){
		switch($action){
			// renders the start template
			case 'start' : {
				echo $this -> view -> render('start.php');		
			} break;
			
			// renders the stop template
			case 'stop' : {
				echo $this -> view -> render('stop.php');	
			} break;
			
			// adds an container to the examoles array
			case 'addToLearner' : {
				
				$_SESSION['view'][$name] = array();
				$_SESSION['view'][$name] = $_SESSION['container'][$name];
				echo $this -> view -> render('start.php');
			} break;
			
			// removes the container again
			case 'remove' :{
				unset($_SESSION['view'][$name]);
				echo $this -> view -> render('start.php');
			} break;
			
			//starts a new Process using the overgiven containers with positive and negatives Examples
			case 'learnThis' : {
				ini_set("soap.wsdl_cache_enabled","1");
			    $this -> dlLearnerConnectionAction();
		        $ids = $this -> getIDsAction();
		        $idss = $ids[0];
		        $ksIDss = $ids[1];
		        $this -> dlLearnerConnectionAction($idss,$ksIDss);
				$posExamples = explode('/', $pos);
				$negExamples = explode('/', $neg);
				$options = explode('/', $options);
				$this->view->startAt = 	date(H .':'. i .':'. s);							
				if( $neg != '' ){
					$negative = array();
					foreach ($negExamples as $key => $value) {
						if ($value !='') {
							$negative[$value] = $_SESSION['container'][$value];
							foreach ($negative[$value] as $key => $val) {
								unset ($negative[$value][$key][2]);
							}
						}
					}
				}
				if( $pos != '' ){
					$positive = array();
					foreach ($posExamples as $key => $value) {
						if ($value !='') {
							$positive[$value] = $_SESSION['container'][$value];
							foreach ($positive[$value] as $key => $val) {
								unset ($positive[$value][$key][2]);
							}		
						}
					}
				}
				$_SESSION['processes'][] = array(date(H .':'. i .':'. s), $positive, $negative);
				
				$posExamplesCont = array();
				$negExamplesCont = array();
				foreach ($_SESSION['processes'] as $key => $value) {
                   foreach ($value as $k => $v){
                     if($k == 1){
	                   foreach ($v as $t => $g){
	                    foreach ($g as $tt => $gg) {
	                     foreach ($gg as $ttt => $ggg){
		                  if ($ttt == 0) array_push($posExamplesCont,$ggg);
		                 }
		                }
	                   }
		             }
	                 elseif($k == 2){
	                   if ($v !=null) {
					   foreach ($v as $t => $g){
	                    foreach ($g as $tt => $gg) {
	                     foreach ($gg as $ttt => $ggg){
		                  if ($ttt == 0) array_push($negExamplesCont,$ggg);
		                 }
		                }
	                   }
				     }
		             }
	               }
	            }
				
				$posE = '';
				foreach ($posExamplesCont as $key=>$value){
				$posE .= $value."<br>" ;
				}
				unset ($_SESSION['processes']);
				if (empty($negExamplesCont)){
				 $negExamplesCont = $this->client->getNegativeExamples($this->id,$this->ksID,$posExamples,count($posExamples),"http://localhost/ontowiki-1/resource");
		         $negExamplesCont = $negExamples->item;
				}
				$curconcept =  $this -> getConceptFromExamplesAction($posExamplesCont,$negExamplesCont);
				$this->view->concept        = $curconcept;
			    $this->view->posE = $posE;
		        $this->view->stopAt = date(H .':'. i .':'. s);
				echo $this -> view -> render('stop.php');
			} break;
			
			// add the learn Sub Class Axiome concept to Model
			case 'addToModelSub' : {
			
			 require_once('Syntax/Manchester/config.php');
			 require_once ('Syntax/Manchester/Parser.php');
			 $stt = str_replace(array(" "), array(null), $st);
		     $a = explode(" ",$stt); 
			 $x='';
             for ($i=1; $i < $argc; $i++) {
	          $x.=$argv[$i].' ';
             }
        
			   if(!$stt=="" || !$x=="" ){
	         
	          $o= new OWLParser();
	       
	          $o->parseString($st);
			 
			  $tempModel = new MemModel();
			  $tempModel = $o->getStructured()->generateRDF();
			  if(sizeof($a) == 1){
			 
			  $tempModel->add(new Statement(new Resource($this->class->getURI()), new Resource("http://www.w3.org/2000/01/rdf-schema#subClassOf"), new Resource($stt)));
			 
			  $this->model->addModel($tempModel);
			  } else if (sizeof($a)>1){
			  $tempModel->add(new Statement(new Resource($this->class->getURI()), new Resource("http://www.w3.org/2000/01/rdf-schema#subClassOf"), Resource("node1")));
			  $this->model->addModel($tempModel);
			 
			  }
			 
			  $tableId = uniqid();
			
			  } 
			  $this->view->actionUrl     = $this->_request->getRequestUri();
			  $this->view->type  = $this->class ? $this->class : $this->resource->getClass();
			  $this->view->tableId       = $tableId;
			  $this->view->model = $tempModel;
			  $this->view->x = $x;
			  $this->view->st = $stt;
              $this->view->title   = "Model for this concept";
			  echo $this->view->render('model.php');
			
			  } break;
			 
			// add the learn Equivalent Class Axiome concept to Model
			case 'addToModelDef' : {
			 require_once('Owl/Structured/Converter.php');
			 require_once('Syntax/Manchester/config.php');
			 require_once ('Syntax/Manchester/Parser.php');
			// $stt = $st;
			$stt = str_replace(array("  "), array(null), $st);
		     $a = explode(" ",$stt); 
			 $x='';
             for ($i=1; $i < $argc; $i++) {
	          $x.=$argv[$i].' ';
             }
        
			   if(!$stt=="" || !$x=="" ){
	         
	          $o= new OWLParser();
	       
	          $o->parseString($st);
			 
			  $tempModel = new MemModel();
			  $tempModel = $o->getStructured()->generateRDF();
			  // if only 1 Resource in the Concept
			  if(sizeof($a) == 1){
			 
			  $tempModel->add(new Statement(new Resource($this->class->getURI()), new Resource("http://www.w3.org/2002/07/owl#equivalentClass"), new Resource($stt)));
			 
			  $this->model->addModel($tempModel);
			  $converter = new Erfurt_Owl_Structured_Converter();
			  $cl= $converter->getEquivalentClasses($this->class);
			  //print_r($cl);
			  foreach ($cl as $k=>$v){
			  echo $v;
			  }
			  } 
			  //more Resources in the Concept
			  else if (sizeof($a)>1){
			  $tempModel->add(new Statement(new Resource($this->class->getURI()), new Resource("http://www.w3.org/2002/07/owl#equivalentClass"), new BlankNode("node1")));
			  $converter = new Erfurt_Owl_Structured_Converter();
			  //$tmp = $converter->convertErfurtModel($tempModel);
			  $this->model->addModel($tempModel);
			  $cl= $converter->getEquivalentClasses($this->class);
			  //print_r($cl);
			  foreach ($cl as $k=>$v){
			  echo $v;
			  }
			  
			  }
			 
			  $tableId = uniqid();
			 
			  } 
			  $this->view->actionUrl     = $this->_request->getRequestUri();
			  $this->view->type  = $this->class ? $this->class : $this->resource->getClass();
			  $this->view->tableId       = $tableId;
			  $this->view->model = $tempModel;
			  $this->view->cl = $cl;
			  $this->view->st = $stt;
              $this->view->title   = "Model for this concept";
			  echo $this->view->render('model.php');
			 
			  } break;
			
			// deletes the selected Process
			case 'delProcess' : {
				unset($_SESSION['processes'][$id]);
				echo $this -> view -> render('exec.php');
			} break;
		    
		  
		}
		}
		// default view
		else{
			echo $this -> view -> render('start.php');
			echo('test');
		}
		
	}
		
}
?>