<?php
/**
 * Semantic Sitemap plug-in controller
 *
 * @author Sebastian Dietzold <sebastian@dietzold.de>
 * @copyright  Copyright (c) 2008, {@link http://aksw.org AKSW}
 * @license    http://opensource.org/licenses/gpl-license.php GNU General Public License (GPL)
 * @version $Id: $
 */
class SitemapController extends OntoWiki_Controller_Action
{
	protected $_store = null;
	protected $_models = null;
	

	/**
	* inits the controller (e.g. for getting the config)
	*
	* @return void
	*/
	public function init()
	{
		parent::init();

		// Disable layout (we want only output xml)
		$this->_helper->layout->disableLayout();

		// Needed here because of the automatic rendering should be deactivated
		$this->_helper->viewRenderer->setNoRender(true);
		
		// Add the path of templates needed by this plugin
		$this->view->addScriptPath(REAL_BASE.'plugins/SemanticSitemap/templates/');
		
		// get the default store from the registry
		$this->_store = Zend_Registry::get('erfurt')->getStore();

		// fetch the modellist from the store
		$this->_models = $this->_store->listModels(false);

		// todo: where to set the content type? currently we get a Quirks mode :-(
		$this->_response->setRawHeader('Content-Type: application/xml');
	}


	/**
	* Generates a semantic sitemap according to Cyganiak et.al. (ESWC2008)
	*
	* @return void
	*/
	public function defaultAction()
	{
		// these dataset items are used by the template
		$datasets = array();
		foreach ($this->_models as $modelUri => $model) {
			$datasets[] = array(
				'datasetURI' => $modelUri,
				'datasetLabel' => $model->getAsResource()->getTitle(),
				'sparqlEndpoint' => $this->config->uriBase . 'service/sparql',
				'sparqlGraphName' => $modelUri,
				'sparqlEndpointLocation' => $this->config->uriBase . 'service/sparql',
				'dataDump' => $this->config->uriBase . 'model/export?output=xml&amp;m=' . urlencode($modelUri)
			);
		}

		// assign view var(s)
		$this->view->datasets = $datasets;
		$this->view->appname = $version = $this->config->version->appname;
		$this->view->version = $version = $this->config->version->number;
		$this->view->versionsuffix = $version = $this->config->version->suffix;


		// render view
		$this->_response->setBody($this->view->render('default.php'));
	}

}

?>
