<?php
/**
 * This plugin implements the integration of social networks such as Facebook and Jabber
 *
 * Socializr only prepares the integration of social networks, the connection to the networks
 * will be established by classes located in the lib folder
 *  
 * @author Thomas Kappel <tkappel@uni-leipzig.de>
 * @copyright Copyright (c) 2008, {@link http://aksw.org AKSW}
 * @license http://opensource.org/licenses/gpl-license.php GNU General Public License (GPL)
 * @version $Id: $
 */
class Socializr extends OntoWiki_Plugin {

  public  $availableNetworks   = false; // available network plugins
  public  $_model              = null;
  public  $_strings            = null;
  public  $_config             = null;
  public  $foaf_fields         = array();
  public  $relations           = array();
  private $message             = "";
  private $networks            = array(); // available networks
  private $accounts            = array(); // accounts of the user
  public  $socializrUri        = 'http://ns.ontowiki.net/plugins/socializr/';
  private $main_window         = "";
  public  $my_name             = "";
  public  $me                  = false;
  
  protected static $_instance  = null;

  public function socializrModel() {
  	$model=$this->_getActiveModel();
  	if (!$model) return false;

		$sparqlstring = 'ASK WHERE {<' . $model->getModelURI() .'> <' . EF_OWL_IMPORTS . '> <' . $this->socializrUri . '>}';
		if ($model->sparqlQuery($sparqlstring) == TRUE) {
			return $model;
		} else {
			return false;
		}
  }

  public function getSocializrDir() {
    return parent::_getPluginRootDir();
  }
  
	protected function _initPlugin($erfurtobject) {
		parent::_initPlugin($erfurtobject);
    $model=$this->socializrModel();
    self::$_instance = $this;
		$this->_strings = Zend_Registry::get('strings');
		if ($model) {
		  $this->_model = $model;
		  require_once('lib/Networks/Abstract.php');

		  // load configuration
      $this->_config = $this->_getPluginConfig()->private;

		  // get available networks
      $counter = 1;
		  while($item = $this->_config->network->get($counter)) {
		    $this->availableNetworks[] = $item->class;
		    $counter++;
		  }
      
		  // load supported foaf fields
      $counter = 0;
      while($item = $this->_config->foaf_field->get($counter)) {
        $this->foaf_fields[] = $item;
        $counter++;
      }
		  
      // load relations
      $counter = 0;
      while($item = $this->_config->relation->get($counter)) {
        $this->relations[$counter] = array(
          "uri"        => $item->uri,
          "label"      => $item->label,
          "symmetric" => $item->transitive
        );
        $counter++;
      }
		  
      $this->load_accounts();
    }
    return true;
  }
  
  public function load_accounts() {
    $model=$this->socializrModel();
    $this->accounts = array(); // reset accounts
	  foreach($this->availableNetworks as $network) {
	    require_once('lib/Networks/'.$network.'/Default.php');
	    $this->networks[$network] = new $network();
	  }
    $sparqlstring = 'PREFIX foaf: <http://xmlns.com/foaf/0.1/> . SELECT ?me, ?name WHERE { ?doc foaf:primaryTopic ?me . ?me foaf:name ?name }';
	  $sparqlquery = $model->sparqlQuery($sparqlstring);
		if(sizeof($sparqlquery)!=1) return;
		$localUser      = current($sparqlquery);
		$me_object      = $localUser["me"];
		$my_name_object = $localUser["name"];
		$this->my_name  = $my_name_object->label;
		$this->me       = $me_object->label;

		// get user accounts
	  $sparqlstring = ' 
      PREFIX foaf: <http://xmlns.com/foaf/0.1/> .
      PREFIX soc: <http://ns.ontowiki.net/plugins/socializr/> .
      PREFIX local: <'.$this->_model->getModelURI().'> .
      SELECT *
      WHERE { local:me foaf:holdsAccount ?account . ?account soc:networkType ?networkType . OPTIONAL { ?account foaf:accountName ?accountName} . OPTIONAL { ?account soc:accountPass ?password} . OPTIONAL { ?account foaf:accountUrl ?accountUrl} }
    ';
    $accounts = $this->_model->sparqlQuery($sparqlstring);
    foreach($accounts as $account) {
			$networkTypeObject = $account["networkType"];
			$networkType       = $networkTypeObject->uri;
			// strip leading model uri
			if(substr($networkType,0,strlen($this->_model->getModelURI())) == $this->_model->getModelURI())
			{
			  $networkType = substr($networkType,strlen($this->_model->getModelURI()));
			}
      
			$accountObject     = $account["account"];
			$accountId         = $accountObject->uri;

      if(!$networkType || !class_exists($networkType)) {
        /*var_dump($networkType);var_dump($accountId);*/
        continue;
      } // own account

			if(isset($account["accountName"])) {
				$accountNameObject = $account["accountName"];
				$accountName       = $accountNameObject->label;
			}
			if(isset($account["password"])) {
				$accountPassObject = $account["password"];
				$accountPassword   = $accountPassObject->uri;
			}
			if(isset($account["accountUrl"])) {
				$accountUrlObject  = $account["accountUrl"];
				$accountUrl        = $accountUrlObject->uri;
			}

      $this->accounts[$accountId] = array(
      	"accountName" => $accountName,
      	"networkType" => $networkType,
      	"network"     => new $networkType(array(
                          "accountId"   => $accountId,
									      	"accountName" => $accountName,
									      	"password"    => $accountPassword,
									      	"accountUrl"  => $accountUrl
      									 )),
      	"password"    => $accountPassword,
      	"accountUrl"  => $accountUrl
      );
    }
//    var_dump(__LINE__);
    return true;
	}

	public function getNetworks() {
	  return $this->networks;
	}
	
	public function getAccounts() {
	  return $this->accounts;
	}

	public function getAccount($id) {
//		var_dump($this->accounts);
    $model=$this->socializrModel();
    // get user accounts
    $sparqlstring = ' 
      PREFIX foaf: <http://xmlns.com/foaf/0.1/> .
      PREFIX soc: <http://ns.ontowiki.net/plugins/socializr/> .
      PREFIX local: <'.$this->_model->getModelURI().'> .
      SELECT *
      WHERE { local:me foaf:holdsAccount ?hold_account . ?hold_account foaf:OnlineAccount ?account . ?account soc:networkType ?networkType . OPTIONAL { ?account foaf:accountName ?accountName} . OPTIONAL { ?account soc:accountPass ?password} . OPTIONAL { ?account foaf:accountUrl ?accountUrl} }
    ';
    $accounts = $model->sparqlQuery($sparqlstring);
    foreach($accounts as $account) {
      $networkTypeObject = $account["networkType"];
      $networkType       = $networkTypeObject->uri;
      $accountObject     = $account["account"];
      $accountId         = $accountObject->uri;

      if(isset($account["accountName"])) {
        $accountNameObject = $account["accountName"];
        $accountName       = $accountNameObject->label;
      }
      if(isset($account["password"])) {
        $accountPassObject = $account["password"];
        $accountPassword   = $accountPassObject->uri;
      }
      if(isset($account["accountUrl"])) {
        $accountUrlObject  = $account["accountUrl"];
        $accountUrl        = $accountUrlObject->uri;
      }

      $this->accounts[$accountId] = array(
        "accountName" => $accountName,
        "networkType" => $networkType,
        "network"     => new $networkType(array(
                          "accountName" => $accountName,
                          "password"    => $accountPassword,
                          "accountUrl"  => $accountUrl
                         )),
        "password"    => $accountPassword,
        "accountUrl"  => $accountUrl
      );
    }
//var_dump($this->accounts[$id]);exit;
	  return $this->accounts[$id];
	}

	public static function id2param($id) {
    $id = str_replace("~","~~",$id);
    $id = str_replace("_","~_",$id);
    $id = str_replace("-","~-",$id);
    $id = str_replace("/","_",$id);
    $id = str_replace("@","-",$id);
	  return $id;
	}
	
	public static function param2id($id) {
    $id = str_replace("-","@",$id);
    $id = str_replace("_","/",$id);
    $id = str_replace("~@","-",$id);
    $id = str_replace("~/","_",$id);
    $id = str_replace("~~","~",$id);
	  return $id;
	}
	
	public function getTempDir() {
    return $this->_getPluginRootDir()."/temp/";
  }
	
	public function addMenu(&$box) {
    $text = (!$this->_model) ? $this->_strings->soc->initialize : $this->_strings->soc->re_initialize;
		// add something to the dropdown menu
    if (isset($box['menu']) && is_array($box['menu']) && 0 < $i = count($box['menu'])) {
      # insert new item before last menu item
      $i-=2;
      if (false !== $pos = strpos($box['menu'][$i],'<li class="last-child">')) {
        $box['menu'][$i] = substr($box['menu'][$i],0,$pos).
        '<li><a href="'.$this->_getOntoWikiBaseUri().'/model/initializeSocializr" class="inline">'.$text.'</a></li>'.
        substr($box['menu'][$i],$pos);
      }
    }
	}
	
	
	/*
	 * adds a ontowiki box to the sidebar
	 */
  public function addBox(&$newWindow)
  {
    if(!$this->_model) return;
		$unsupportedNetworks=array();
    $accounts_out       = "";
    $networks_out       = "";
    $assignAccounts_out = "";
		foreach($this->accounts as $accountId=>$account) {
			$network = $account["network"];
			$about_network = call_user_func("about" . $account["networkType"]);
			$human_readable_network_name = $about_network["name"];
			$is_connected = $network->connected();
      $switch_state_link = ($network->require_connect_button() || ($is_connected && $network->supports("disconnect") ))
        ? '<a href="'.$this->_getOntoWikiBaseUri().'/model/'.($is_connected?'disconnect':'connect').'/account/'.$this->id2param($accountId).'/" class="replace-inline"><img alt="" src="'.$this->_getOntoWikiBaseUri().'/plugins/Socializr/images/icon_power.png" title="'.($is_connected ? $this->_strings->soc->networks->disconnect : $this->_strings->soc->networks->connect).'" /></a>'
        : '';
      $accounts_out.='
        <li>
          '.$account["accountName"]." @&nbsp;".$human_readable_network_name.'
          '.$switch_state_link // /*
          .'
          <a href="'.$this->_getOntoWikiBaseUri().'/model/editAccount/account/'.$this->id2param($accountId).'/" class="replace-inline"><img alt="" src="'.$this->_getOntoWikiBaseUri().'/themes/default/images/icon-edit.png" title="'.$this->_strings->soc->networks->editAccount.'" /></a>
          ' // */
          .($is_connected ? '<a href="'.$this->_getOntoWikiBaseUri().'/model/fetchcontacts/account/'.$this->id2param($accountId).'/" class="replace-inline"><img alt="" src="'.$this->_getOntoWikiBaseUri().'/plugins/Socializr/images/icon_refresh.png" height="16" title="'.$this->_strings->soc->fetchContacts->fetchNewContacts.'" /></a>' : '').'
        </li>
      ';
		}
		if(strlen($accounts_out)) {
			$accounts_out='<li>'.$this->_strings->soc->fetchContacts->fetchNewContacts.':<ul>'.$accounts_out.'</ul></li>';
		} else $accounts_out="";

    foreach($this->networks as $name=>$Network) {
      $about_network = call_user_func("about" .$name);
      $human_readable_network_name = $about_network["name"];
    	$networks_out.='
          <li>
            <a href="'.$this->_getOntoWikiBaseUri().'/model/addAccount/network/'.$name.'/" class="replace-inline">'.$human_readable_network_name.'</a>
          </li>';
    }
    if(strlen($networks_out)) {
			$networks_out='
			<li><a onclick="Element.toggle(\'socializr_network_list\');">'.$this->_strings->soc->networks->addAccount.'</a>
			  <ul id="socializr_network_list" style="display:none">'.$networks_out.'</ul>
			</li>';
    } else $networks_out="";
		
    if(strlen($accounts_out)>0) {
      $assignAccounts_out.='
          <li>
            <a href="'.$this->_getOntoWikiBaseUri().'/model/assignAccounts" class="replace-inline">'.$this->_strings->soc->fetchContacts->manageContacts.'</a>
          </li>';
      $assignAccounts_out.='
          <li>
            <a href="'.$this->_getOntoWikiBaseUri().'/model/fetchUnknownAccounts" class="replace-inline">'.$this->_strings->soc->networks->synchronize.'</a>
          </li>';
    }
    $output='<ul>'.$accounts_out.$networks_out.$assignAccounts_out.'</ul>';
    $content = array(
			'title'   => $this->_strings->soc->Socializr,
			'content' => $output);
    $newWindow .= $this->_view->partial('micro/window.php', $content);
    return true;
  }
  
  public function generatePersonIdentifier($contact) {
		$sparql_person = 'PREFIX foaf: <http://xmlns.com/foaf/0.1/> .
    ASK WHERE {?global foaf:Person ?person . ?person foaf:name <'.$contact["name"].'>}';
		$name = "";
		// allow only standard letters (a-z) for URI
		for($pos=0;$pos<strlen($contact["name"]);$pos++) {
		  $char = $contact["name"][$pos];
		  $name.= preg_match("/[a-z]/i",$char) ? $char : "_";
		}
    if(!$this->_model->sparqlQuery($sparql_person)) {
    	return $name;
    } else return $name.'-'.mktime();
  }
  
  public function getAccountUri ($accountId, $networkName) {
    return $this->_model->getModelURI()."accounts/".$networkName."/".str_replace('_', '', str_replace('~', '', $accountId));
  }
  
  public function getPersonUri ($personId) {
    return $this->_model->getModelURI()."contacts/".$personId;
  }
  
  public function addPerson2Model($contact, $network, $knows_another_person = false) {
    // first we check, if this account is already known, else we should create a new person with this account
    $account_uri = $this->getAccountUri($network->getAccountIdentifier($contact), $network->_Name);
    $returns = $account_uri;
    $sparql_account = 'PREFIX foaf: <http://xmlns.com/foaf/0.1/> .
      ASK WHERE {?account foaf:OnlineAccount <'.$account_uri.'>}';

    if(!$this->_model->sparqlQuery($sparql_account)) { // account does not exist

    	$n3_prefixes = '
        @prefix foaf: <http://xmlns.com/foaf/0.1/> .
        @prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#> .
        @prefix soc: <http://ns.ontowiki.net/plugins/socializr/> .
        ';
			
      $about_network = call_user_func("about" .$network->_Name);
      $human_readable_network_name = $about_network["name"];
    	
    	$n3_contact = '
        <'.$account_uri.'> a foaf:OnlineAccount;
          rdfs:label "'.$human_readable_network_name.$this->_strings->soc->fetchContacts->NetworkAccountOf.' '.$contact["name"].'";
          foaf:accountServiceHomepage <'.$network->getServiceHomepage().'>;
          soc:networkType "'.$network->_Name.'";
          foaf:accountName "'.$contact["accountName"].'". 
			';
			// Wenn keine Person mit dem Namen des neuen Kontaktes existiert, wird sie angelegt => später sollen Korrekturen möglich sein
      // compare with known contacts, add person if unknown
      $existing_person = $this->findMetaContact($contact);
      if(!$existing_person) {
      	$person_uri = $this->getPersonUri($this->generatePersonIdentifier($contact));
        $n3_person = '
        <'.$person_uri.'> a foaf:Person;
          rdfs:label "'.$contact["name"].'"';
        $n3_person.= '.';
        $n3_contact .= $n3_person; 
		    $returns = $person_uri;
      } else {
        $person_uri = $existing_person;
      }
			$n3 = $n3_prefixes.$n3_contact;

			// it's be better to pass the n3 string directly
      $use_file = false;
      if($use_file) { 
  			$n3_filename = $this->_getPluginRootDir()."/temp/temp.n3";
  			if(!is_writable($n3_filename)) return false;
  			$temp=fopen($n3_filename,"w");
  			fputs($temp,$n3);
  			fclose($temp);
        $n3 = $n3_filename;
      }
      // */
      $newModel = new MemModel();
      $newModel->loadFromString($n3, 'n3');
      $this->_model->addModel($newModel);

      $this->_model->add(
        new Statement(
          $this->_model->resourceF($person_uri),
          new Resource("http://xmlns.com/foaf/0.1/holdsAccount"),
          $this->_model->resourceF($account_uri)
        )
      );      
      
      if(!$knows_another_person) {
  	    $sparqlstring        = 'PREFIX foaf: <http://xmlns.com/foaf/0.1/> . SELECT ?person WHERE { ?doc foaf:primaryTopic ?person . ?person foaf:name ?name }';
  		  $sparqlquery         = $this->_model->sparqlQuery($sparqlstring);
  			if(sizeof($sparqlquery)!=1) return;
  			$localUser           = current($sparqlquery);
  			$relevant_person     = $localUser["person"];
        $relevant_person_uri = $relevant_person->uri;
      } else {
        $relevant_person_uri = $knows_another_person;
      }
        
			if(isset($person_uri))
      $this->_model->add(
                    new Statement(
                        $this->_model->resourceF($relevant_person_uri),
                        new Resource("http://xmlns.com/foaf/0.1/knows"),
                        $this->_model->resourceF($person_uri)
                    )
                );
    }
    $person = $this->_model->resourceF($person_uri);
    foreach($contact as $field=>$contact_data){
      if(in_array($field,$this->foaf_fields) && !$person->getPropertyValue("http://xmlns.com/foaf/0.1/" . $field) && strlen(trim($contact_data))) {
        $person->setPropertyValue("http://xmlns.com/foaf/0.1/" . $field, new Literal($contact_data));
      }
    }  
    return $returns;
  }
  
	public function findMetaContact($newContact) {
   /*
    * accounts belong to one contact if these are equal:
    *  - OpenID or
    *  - e-mail or
    *  - depiction uri or
    *  - instant messaging account or
    *  - first- and last name (could be reduced to:
    *    - first- and last name AND
    *      - homepage or
    *      - telephone number
    *    )
    */
		$conditions=array();
		/// for testing only (same name needs not to mean the same person):
		if(isset($newContact["name"]))         $conditions[]='?uri foaf:name "'        .$newContact["name"]        .'"';
//		$conditions[]='?x foaf:givenname "'.$newContact["givenname"].'" . ?x foaf:family_name "'.$newContact["family_name"];

		if(isset($newContact["openid"]))       $conditions[]='?uri foaf:openid "'      .$newContact["openid"]      .'"';
		if(isset($newContact["depiction"]))    $conditions[]='?uri foaf:depiction "'   .$newContact["depiction"]   .'"';
		if(isset($newContact["mbox_sha1sum"])) $conditions[]='?uri foaf:mbox_sha1sum "'.$newContact["mbox_sha1sum"].'"';

		if(isset($newContact["givenname"]) && isset($newContact["family_name"])) {
			$sub_conditions = array();
			if(isset($newContact["homepage"])) $sub_conditions[]='?uri foaf:homepage "'.$newContact["homepage"].'"';
			if(isset($newContact["phone"]))    $sub_conditions[]='?uri foaf:phone	"'.$newContact["phone"].'"';
			if(sizeof($sub_conditions)) $conditions[]='?uri foaf:givenname "'.$newContact["givenname"].'" . ?uri foaf:family_name "'.$newContact["family_name"].'" . {'.implode('} UNION {',$sub_conditions).'}';
		}
		// first let's try a single query, it will only something if everything matches
	  $sparqlstring = 'PREFIX foaf: <http://xmlns.com/foaf/0.1/>
      SELECT ?uri, ?name where { ?uri foaf:name ?name . { {
		    '.implode('} UNION {', $conditions).'
		  } } }';
		$meta_contacts = $this->_model->sparqlQuery($sparqlstring);
		if(sizeof($meta_contacts)==1) {
		  $meta_contact = current($meta_contacts);
		  $meta_contact_uri_object  = $meta_contact["uri"];
		  $meta_contact_uri         = $meta_contact_uri_object->uri;
		  $meta_contact_name_object  = $meta_contact["name"];
		  $meta_contact_name         = $meta_contact_name_object->label;
		  $meta_contact_name = $meta_contact["name"]->label;
//			$output = ' ' . $this->_strings->soc->fetchContacts->isKnown.': <span content="'.$this->_getOntoWikiBaseUri().'/socializr/Sebastian Dietzold" property="http://xmlns.com/foaf/0.1/knows"><span class="icon-button expand"/><a class="expandable Resource" href="'.$this->_getOntoWikiBaseUri().'/resource/view/?r='.urlencode($meta_contact_uri).'" about="'.urlencode($meta_contact_uri).'">'.$meta_contact_name.'</a></span><a href="'.$this->_getOntoWikiBaseUri().'/resource/view/socializr/mergeContact/network/'.$networkName.'/uid/'.$newContact['uid'].'/local_name/'.urlencode($meta_contact_name).'/?r='.$this->_model->getModelURI().'">'.$this->_strings->soc->fetchContacts->mergeContact.'!</a>';
//		  var_dump($meta_contact_name);exit;
			return $meta_contact_uri;
		} elseif(!sizeof($meta_contacts)) {
			return false;// new contact
		} else {
			return false;// not sure
		}
	}

	public function assignPersons($person1_uri, $person2_uri, $relation_uri) {
    $this->_model->add(
      $this->_model->resourceF($person1_uri),
      new Resource($relation_uri),
      $this->_model->resourceF($person2_uri)
    );

    $this->_model->add(
      $this->_model->resourceF($person2_uri),
      new Resource($relation_uri),
      $this->_model->resourceF($person1_uri)
    );
  }
  
  public function releaseMergedPersons($person1_uri, $person2_uri, $relation_uri) {
    $this->_model->remove(
      $this->_model->resourceF($person1_uri),
      new Resource($relation_uri),
      $this->_model->resourceF($person2_uri)
    );

    $this->_model->remove(
      $this->_model->resourceF($person2_uri),
      new Resource($relation_uri),
      $this->_model->resourceF($person1_uri)
    );
  }

  /**
   * return names of searchable networks, where this person may have an account we don't know, yet
   */
  public function getNetworksWithPossiblyUnknownAccounts($person_uri) {
    $candidates = array();
    foreach($this->getAccounts() as $account) {
      $network = $account["network"];
      if(!$network->supports("search_members") && !$network->supports("yasni")) continue;
      $networkName = $network->_Name; 
      $sparqlstring = ' 
        PREFIX foaf: <http://xmlns.com/foaf/0.1/> .
        PREFIX soc: <http://ns.ontowiki.net/plugins/socializr/> .
        ASK WHERE { <'.$person_uri.'> foaf:holdsAccount ?account . ?account soc:networkType "'.$networkName.'" }
      ';
      if(!$this->_model->sparqlQuery($sparqlstring)) $candidates[]=$networkName;
    }
    return $candidates;
  }
  
  public static function getInstance() {
      if (null === self::$_instance) return false;
      return self::$_instance;
  }

	public function getOntoWikiBaseUri() {
	  return $this->_getOntoWikiBaseUri();
	}
 	
	public function grabSocializrActions() {
		$frontController = Zend_Controller_Front::getInstance();
		$request         = $frontController->getRequest();
		$controllerName  = strtolower($request->getControllerName());
		$action          = strtolower($request->getActionName());

		if ($controllerName === 'model') {
			$network   = $request->get('network');
			$account   = $this->param2id($request->get('account'));
			
			  	//var_dump($this->socializrModel());exit;
  		if(!$this->_model && $action!="initializesocializr" && $action!="socializrupdate") return;
  		
			if($action) switch($action) {
        // actions requiring a specified account
        case("clearaccount"):
        case("editaccount"):
		    case("fetchcontacts"):
		    case("fetchcontact"):
			  	if($account===false) return;
					$frontController->addControllerDirectory(dirname(__FILE__));
					$request->setControllerName('socializr');
					$request->setDispatched(false);
			  	break;
        // actions requiring a specified network
	      case("addaccount"):
	      case("ignoreaccount"):
        case("message"):
			  	if(!$network) return;
					$frontController->addControllerDirectory(dirname(__FILE__));
					$request->setControllerName('socializr');
					$request->setDispatched(false);
					break;
        // actions requiring a specified account and network
        case("connect"):
        case("disconnect"):
        case("fetchunknownpersondetails"):
			  	if($account===false && !$network) return;
					$frontController->addControllerDirectory(dirname(__FILE__));
					$request->setControllerName('socializr');
					$request->setDispatched(false);
					break;
        // actions requiring something else specified in the controller 
        case("assignaccounts"):
        case("assignaccount"):
        case("fetchunknownaccounts"):
        case("findaccounts"):
        case("initializesocializr"):
        case("listaccounts"):
		  	case("socializrupdate"):
					$frontController->addControllerDirectory(dirname(__FILE__));
					$request->setControllerName('socializr');
					$request->setDispatched(false);
          break;
			}
		} 
	}

  public function addIncludes(&$includes){
		$includes.='<!-- Socializr scripts -->'. PHP_EOL;
		$includes.='<script type="text/javascript" src="'.$this->_getPluginBaseUri().'/../Socializr/jquery.form.js"></script>'.PHP_EOL;
		$includes.='<script type="text/javascript" src="'.$this->_getPluginBaseUri().'/Socializr.js"></script>'.PHP_EOL;
    //$includes.='<script type="text/javascript" src="'.$this->_getPluginBaseUri().'/../ContainerManager/ContainerManager.js"></script>'.PHP_EOL;
		$includes.='<!-- End of Socializr scripts -->'.PHP_EOL;
		return true;
	}
}

?>
