<?php
define('SWORE_REQUIREMENT', 'http://ns.softwiki.de/req/Requirement');
define('SWORE_ISREQUIREMENTOF', 'http://ns.softwiki.de/req/isRequirementOf');
define('SWORE_ABSTRACT_REQUIREMENT','http://ns.softwiki.de/req/AbstractRequirement');
define('SWORE_MODEL', 'http://ns.softwiki.de/req/');
define('SWORE_FILE_PROPERTY', 'http://ns.softwiki.de/req/hasLinkedFile');
define('SWORE_SOLUTION_PROPERTY', 'http://ns.softwiki.de/req/solution');
define('SWORE_FAVSOLUTION_PROPERTY', 'http://ns.softwiki.de/req/solutionFavorite');
define('SWORE_EXSOLUTION_PROPERTY', 'http://ns.softwiki.de/req/solutionExcluded');


/**
 * SoftWiki plugin main class.
 *
 * @author Leszek Kotas <Leszek.Kotas@gmail.com>
 * @version $Id: SoftWiki.php 2768 2010-03-12 17:35:32Z triechert $
 */
class SoftWiki extends OntoWiki_Plugin {
    
    private $_config = null;
    private $_session = null;
    private $_model = null;
    private $_store = null;
    private $_sworeModel = false;
    private $_sworeRequirement = false;
    private $_strings = false;
    private $_ac = null;

    protected function _initPlugin($erfurtobject) {
        
        parent::_initPlugin($erfurtobject);

        return true;
    }

    public function isSwore(){
        
        $this->_config  = Zend_Registry::get('config');
        $this->_session = Zend_Registry::get('session');
        $this->_store   = Zend_Registry::get('erfurt')->getStore();
        $this->_strings = Zend_Registry::get('strings');
        $this->_ac      = Zend_Registry::get('ac');
        
        if ($this->_config->activeModel) {
            // set requirement Flag to false
            $this->_sworeRequirement = false;

            #get activeModel and compare with SWORE
            $this->_model = $this->_config->activeModel;
            $sparqlstring   = 'ASK WHERE { <' . $this->_model->getModelURI() .'> <' . EF_OWL_IMPORTS . '> <' 
                            . SWORE_MODEL . '> }';
            $this->_sworeModel = $this->_model->sparqlQuery($sparqlstring);

            // if its a swore Model, than activate sworeModel Flag
            if ($this->_sworeModel) {
                $resourceUri = $this->_session->activeResourceUri;
                $sparqlstring2  = 'ASK WHERE {<' . $resourceUri .'> <' . EF_RDFS_NS .'subClassOf> <' 
                                . SWORE_REQUIREMENT . '>}.';
                // if its a Requirement, than activate sworeReqirement Flag
                if ($this->_model->sparqlQuery($sparqlstring2) || $resourceUri === SWORE_REQUIREMENT) {
                    $this->_sworeRequirement = true;
                }
            }
        }
        
        return true;
    }

    /**
     * adds SoftWiki.js for javascript usage
     *
     */
    public function adIncludes(&$includes){

        $includes  .=  '<!-- SoftWiki scripts -->' . PHP_EOL
                    .   '<script type="text/javascript" src="' . $this->_getPluginBaseUri() 
                    .   '/jquery.form.js"></script>' . PHP_EOL
                    .   '<script type="text/javascript" src="' . $this->_getPluginBaseUri()
                    .   '/SoftWiki.js"></script>' . PHP_EOL
                    .   '<script type="text/javascript" src="'.$this->_getOntoWikiBaseUri().'/lib/Erfurt/public/plugins/widgets/htmlEdit'
                    .   '/fckeditor/fckeditor.js"></script>' . PHP_EOL
                    .   '<!-- End of SoftWiki scripts -->' . PHP_EOL;
                    
        return true;
    }

    /**
     * places the SoftWiki Side windows.
     */
    public function addSoftWikiSideToView(&$content){

        $this->checkView();
        
        if ($this->_sworeModel) {
            $mSelection = $this->mainSelectionContent();
             
            if ($mSelection !== false) {
                $content .= $this->_view->partial('micro/window.php', $mSelection);
            }
        }
        
        return true;
    }

    /**
     * Creates the SoftWiki Themen window.
     * @return array
     */
    public function mainSelectionContent() {
     
        // model has to be selected
        if (!$this->_sworeModel) {
            return false;
        }
         
        $content   = false;
         
        $auth      = Zend_Auth::getInstance();
        $identity  = $auth->getIdentity();
         
        $isAnonymousUser   = ($identity['anonymous'] === true)                                      ? true : false;
        $isModelEditable   = ($this->_ac->isModelAllowed('edit', $this->_model->getModelURI()))     ? true : false;
        $isOntoWikiAllowed = ($this->_ac->isActionAllowed('SoftWikiUseOntoWiki'))                   ? true : false;
        
        $content = '<div>'
                 . '<a href="'.$this->_config->uriBase . 'softwiki/mselect">' 
                 . '<img style="margin:0.5em; display:block; margin-left:auto; margin-right:auto;" src="' 
                 .  $this->_config->uriBase . 'plugins/SoftWiki/sw-logo.png"/>'
                 . '<h4 style="text-align:center;"></a>';
// Here we should read the label of the model to show the Title 
		if (isset($_SESSION['swint_title'])) {
        	$content = $content . $_SESSION['swint_title']; 
		} else {
			$content = $content . $this->_strings->sw->common->description; 
		}
        $content = $content = $content . '</h4>'
                 . '</div>';
       
		if (!$isAnonymousUser) {
			$content .= '<p>'
					 .  $this->_strings->sw->mainselect->loggedinas . ': ' .  $identity['username']
					 .  '</p>';

        } else {
            //$content .= '<p class="messagebox info"><a href="#login">' . $this->_strings->sw->mainselect->loginplease . '</a></p>';
            //$content .= '<p class="messagebox info"><a href="'.$ow_folder.'wiki/registernew/">' . $this->_strings->sw->mainselect->registerplease . '</a></p>';
        }
	$ow_folder=substr($_SERVER["PHP_SELF"],0,-8);
	
	$content .= '<p><a href="'.$ow_folder.'softwiki/mselect">'.$this->_strings->sw->mainselect->selectproject.'</a></p>';
 
        // only show <select> if user has the right to use the regular ontowiki
        if ($isOntoWikiAllowed) {
            $resource    = $this->_model->resourceF(SWORE_REQUIREMENT, false);
            $stakeholder = $this->_model->resourceF('http://ns.softwiki.de/req/Stakeholder', false);
            
            $content .= '<form class="row-input"><div class="no-min-width">'
                    . '<label class="onlyAural" for="mainselection">Main Selection</label>'
                    . '<select name="mainselection" id="mainselection" size="1" onChange="go(this)" class="width99">'
                    . '<option>' . $this->_strings->sw->mainselect->preselect . '</option>'
                    . '<option ' 
                    . ((isset($_GET['r']) && ($_GET['r'] == SWORE_REQUIREMENT)) ? 'selected="selected" ' : '') 
                    . 'value="' . $this->_config->uriBase 
                    . 'resource/list/?r=http://ns.softwiki.de/req/Requirement">' 
                    . $resource->getTitle($this->_session->modelLanguage) . '</option>'
                    . '<option value="' . $this->_config->uriBase 
                    . 'softwiki/admin">' 
                    . 'SoftWiki Administration' . '</option>'
                    . '<option value="' . $this->_config->uriBase
                    . 'resource/list/skos%3AConcept' .'">' 
                    . 'Topics'. '</option>'
                    . '<option value="' . $this->_config->uriBase
                    .  'resource/list/foaf%3AAgent?m=http%3A%2F%2Flocalhost%2FOntoWiki%2FConfig%2F' .'">' 
                    . 'User'. '</option>'
                    . '<option value="' . $this->_config->uriBase
                    . 'wiki/registernew/' .'">' 
                    . 'Register New User' . '</option>'
                    . '</select></div>';

        }
		
         
// Thomas Riechert; this code is useful in combination with the softwiki-integrator Plugin
// swint_return contains an URL to the calling application
		if(isset($_SESSION['swint_return'])){
			// Some special TEST to get the refering Application
			if (strpos($_SESSION['swint_return'], "/project.php?id")===false) $refapp=" "; else $refapp=" COfundOS Project ";
			$content = $content
                    . '<a href="' . $_SESSION['swint_return'] . '" id="swLogout" class="button">'
                    . '<img alt="" src="' . $this->_config->uriBase . 'themes/default/images/icon-logout.png"/> ' 
                    . $this->_strings->sw->backtobutton.$refapp.$_SESSION['swint_projectid'].'</a>';
		} else {

	        // only show the logout button if the current user is not the anonymous user
    	    	if (!$isAnonymousUser) {
        		    $content = $content
            		        . '<a href="' . $this->_config->uriBase . 'wiki/logout/" id="swLogout" class="button">'
                		    . '<img alt="" src="' . $this->_config->uriBase . 'themes/default/images/icon-logout.png"/> ' 
        		            . $this->_strings->sw->logoutbutton . '</a>';
        		}
		}
		
        // only show the new requirement button if the current user has edit rights for the model
        if ($isModelEditable)  {
            $content = $content
                    . '<a style="display: none;" id="swNew" class="button" title="'
                    . $this->_strings->sw->tooltip->newreqbutton . '">'
                    . '<img alt="" src="' . $this->_config->uriBase . 'themes/default/images/icon-add.png"/> '
                    . $this->_strings->sw->newreqbutton . '</a>';


	        // Simulate "Add New Requirement"-button as an work around if Softwiki started by using the open Method
	        if (strpos($_SERVER['REQUEST_URI'],"softwiki/open")>0)  {
	            $content = $content
	                    . '<a href="'.$this->_config->uriBase.'resource/list/?r=http://ns.softwiki.de/req/Requirement" class="button" title="'
	                    . $this->_strings->sw->tooltip->newreqbutton . '">'
	                    . '<img alt="" src="' . $this->_config->uriBase . 'themes/default/images/icon-add.png"/> '
	                    . $this->_strings->sw->newreqbutton . '</a>';
	        }

        }

	// Get current Project

	$doapProjectTitle=$this->_strings->sw->mainselect->title;
	$store = Zend_Registry::get('erfurt')->getStore();
	$doapProjectSQL = "SELECT DISTINCT subject FROM statements WHERE predicate = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#type' AND object = 'http://usefulinc.com/ns/doap#Project' AND modelID = " . $this->_model->getModelID();
	$resultData = $store->sqlQuery($doapProjectSQL);
	$doapProjectURI="";	
	foreach ($resultData as $doapProject) {
		$doapProjectURI = $doapProject[0];
		break;			
	}
	if ($doapProjectURI!="") {
		$doapProjectTitle = $this->_model->resourceF($doapProjectURI, false) ->getTitle($lang); 		
	}

        // if $content is not equal to false at this point, then there is something to show... so add the missing
        // data for the box (title, close the form)
        if ($content !== false) {
            $content .= '</form>';
             
            $retArray = array(
                'title'     => $doapProjectTitle,
                'content'   => $content,
                'cssId'     => 'swMainselectionbox'
            );
            
            return $retArray;
        }
        // else return false for there is nothing to show
        else {
            return false;
        }
    }
    
    /**
     * Modifies the OntoWiki dispatch process to forward certain actions
     * to the SoftWiki controller, omitting default OntoWiki actions.
     */
    public function grabActions() {
        
        // get front controller
        $frontController = Zend_Controller_Front::getInstance();

        // get request object
        $request = $frontController->getRequest();

        // save controller and action names
        $controller = $request->getControllerName();
        $action     = $request->getActionName();


        if ($controller == 'softwiki'){
            // add the plug-in dir as a controller dir
            $frontController->addControllerDirectory(dirname(__FILE__));
        }
		
        if($this->_sworeModel && $this->_sworeRequirement){
            // get front controller
            $frontController = Zend_Controller_Front::getInstance();

            // get request object
            $request = $frontController->getRequest();

            // get selected resource's URI
            $session     = Zend_Registry::get('session');
            $resourceUri = $session->activeResourceUri;

            // get erfurt store
            $store = Zend_Registry::get('erfurt')->getStore();

            // save controller and action names
            $controller = $request->getControllerName();
            $action     = $request->getActionName();

            // only dispatch those actions to the SoftWiki controller that it can handle
            // all other actions should be handled by the default OntoWiki controller
            if ($controller == 'resource' && $action == 'view') {

                // add the plug-in dir as a controller dir
                $frontController->addControllerDirectory(dirname(__FILE__));
                // set dispatched to false so that default OntoWiki action will not be dispatched
                $request->setDispatched(false);
                // set the controller to softwiki
                $request->setControllerName('softwiki');
                // set the action to view
                $request->setActionName('view');
            }

            if ($controller == 'resource' && $action == 'list') {

                // add the plug-in dir as a controller dir
                $frontController->addControllerDirectory(dirname(__FILE__));
                // set dispatched to false so that default OntoWiki action will not be dispatched
                $request->setDispatched(false);
                // set the controller to softwiki
                $request->setControllerName('softwiki');
                // set the action to view
                $request->setActionName('list');
            }

        }
        
        // redirect if ontowiki user isn't allowed to open ontowiki pages
        $auth      = Zend_Auth::getInstance();
        $identity  = $auth->getIdentity();
        
        $forbiddenControllers = array('index','model');
        if (in_array($controller,$forbiddenControllers) &&
        !$this->_ac->isActionAllowed('SoftWikiUseOntoWiki') &&
        ($identity['anonymous'] !== true) ) {
        		// init check vars
        		$this->isSwore();
        		if ($this->_sworeModel) {
        			// need to do this via http header because other redirect isn't available
        			// before dispatchment has taken place
        			$header = 'Location: ' .
					$this->_config->uriBase .
					'softwiki/list/?r=' .
					urlencode(SWORE_REQUIREMENT);
					header($header,null,303);
					exit();
        		}
        }
    }

    /**
     * check for view object
     */
    public function checkView() {
        // get action controller object
        if (isset($this->_pluginManager->action)) {
            $this->_action = $this->_pluginManager->action;
        }

        // get view object from action controller
        if ($this->_action) {
            $this->_view = $this->_action->view;
        }

        // add plug-in basedir as script path
        if ($this->_view) {
            $this->_view->addScriptPath($this->_getPluginRootDir());
        }
    }

    /**
     * Redirects to the softwiki/list action when a model is selected.
     *
     * @param string original OntoWiki login redirect
     */
    public function redirectOnSelectModel(&$redirectUrl) {
        // init check vars
        $this->isSwore();

        // redirect if we have a model importing swore
        if ($this->_sworeModel) {
            $redirectUrl = $this->_config->uriBase . 'softwiki/list/?r=' . urlencode(SWORE_REQUIREMENT);
        }
    }

    /**
     * Method called after login redirects to requirements-view or
     * to /softwiki/mselect if multiple models exists
     *
     * @param string $redirectUrl
     */
    public function redirectOnLogin (&$redirectUrl) {

        //echo 'SoftWiki-> redirectOnLogin'; exit();

        //Make session and log available
        $session = Zend_Registry::get('session');
        $log = Zend_Registry::get('owLog');
        
        // render knowledbe bases box (unspecified)
        //$session -> modelsBoxDisabled = true;
        
        $session -> modelsBoxMinimized = true;
        $session -> searchBoxMinimized = true;
		

        // Check view to access view object
        $this -> checkView();

        // get all allowed models for the current user
        $models = $this -> _erfurtApp -> getStore() -> listModels();
        $sworeModels = array();
        $sworeModelUris = array();
		
        //check every with a short sparql if it is a swore-model
        foreach ($models as $model) {
            $sparqlstring = 'ASK WHERE {<' . $model->getModelURI() .'> <' . EF_OWL_IMPORTS . '> <' . SWORE_MODEL . '>}';
            if ($model -> sparqlQuery($sparqlstring)) {
                $sworeModels[] = $model;
                $sworeModelUris[] = $model -> modelURI;
            }
        }

        // Take actions if we have min. 1 swore-model
        if (sizeof($sworeModels) != 0) {

            $frontController = Zend_Controller_Front::getInstance();
            $request = $frontController->getRequest();

            // if we have multiple models, then do selection screen
            // sometimes user wish to start with an initial screen, where a model has to be selcted
            if ((sizeof($sworeModels) != 1) or (true)){
				$header = 'Location: ' . $this->_getOntoWikiBaseUri().'/softwiki/mselect?from='.json_encode($sworeModelUris);
				header($header,null,303);
				exit();
            } else {
                // Select the only available model
                $session -> activeModel = $sworeModels[0];
                $session -> activeModelUri = $session->activeModel->modelURI;
                $log->debug('Model selected: <' . $session->activeModel->modelURI . '>');

                // redirect to the requirements list
                $header = 'Location: ' . $this->_getOntoWikiBaseUri().'/resource/list/?r='.urlencode(SWORE_REQUIREMENT);
                header($header,null,303);
                exit();
            }
        } else {
            // do nothing
        }
    }
}
?>
