<?php

/**
 * SoftWiki plug-in controller
 *
 * @author Norman Heino <norman.heino@gmail.com>
 * @version $Id: SoftwikiController.php 2773 2010-09-14 21:51:33Z triechert $
 */
class SoftwikiController extends OntoWiki_Controller_Action {

	/**
	 * Enter description here...
	 *
	 * @var unknown_type
	 */
	private $_maxErgs = 10;

	public function init()
	{
		parent::init();

		$this->view->headScript()->appendScript('var allFields ="' . $this->strings->sw->allFields . '"');
	}

	/**
	 * SoftWiki view action.
	 * Overwrites the default OntoWiki view action and renders its own templates.
	 */
	public function viewAction() {
		// TODO fetch property values with one sparql query???
		// TODO language support for all props???

		// --------------------------------------------------------------------
		// --- Do some initializing stuff -------------------------------------
		// --------------------------------------------------------------------

		$this->_strings = Zend_Registry::get('strings');


		// as a service it needs no view renderer and no layout template
		$this->_helper->viewRenderer->setNoRender();
		$this->_helper->layout->disableLayout();

		$params = $this->_request->getParams();
		$session = Zend_Registry::get('session');
		$lang = $session->modelLanguage;

		// stripslashes for all values
		if (get_magic_quotes_gpc()) {
			foreach ($_GET as $key => $value) {
				$params[$key] = stripslashes($value);
			}
		} else $params = $_GET;

		// --------------------------------------------------------------------
		// --- New Requirement ------------------------------------------------
		// --------------------------------------------------------------------

		if (isset($this->_request->new)) {
			// fetch the topic and tag uris from params
			if (!isset($params['resources'])) {
				$uris = array();
			} else {
				$uris = json_decode($params['resources']);
			}

			// drop requirement uris
			$newUris = array();
			foreach ($uris as $uri) {
				if (!($uri === SWORE_REQUIREMENT)) {
					$newUris[] = $uri;
				}
			}
			$uris = $newUris;

			// get store and model id
			$store = Zend_Registry::get('erfurt')->getStore();
			$modelID = $this->model->getModelID();

			// initialize the tags array
			$tags = array();
			$topics = array();

			foreach ($uris as $u) {
				$r = $this->model->resourceF($u, false);
				$type = $r->getPropertyValue(EF_RDF_TYPE);

				if (($type instanceof Erfurt_Rdfs_Resource)
				&& ($type->getURI() === 'http://www.w3.org/2004/02/skos/core#Concept')) {

					$topics[] = $r;
				} else {
					$tags[] = $r->getTitle($lang);
				}
			}

			//write all tags into the output string
			$tagsString = join(', ', $tags);
		}


		if (isset($this->_request->r)) {
			$resource = $this->model->resourceF($params['r']);
		}


		// --------------------------------------------------------------------
		// --- Find Files for the "Files" Innerwindow -------------------------
		// --------------------------------------------------------------------

		if (isset($this->_request->r)) {
			$query = 'SELECT * WHERE { <' . $resource -> getURI() . '> <' . SWORE_FILE_PROPERTY . '> ?file' .
            ' . ?file <' . EF_RDFS_LABEL . '> ?label}';
			$qr = $this -> model -> sparqlQuery($query);
			if (is_array($qr)) {
				foreach ($qr as $f) {
					//echo '<!-- File: ' . $f['file']->getURI() . ' --!>';
					$temp = array(
    					'uri' => $f['file']->getURI(),
    					'label' => $f['label']->getLabel());
					$files[] = $temp;
				}
			}
			$this -> view -> files = $files;
		}


		// --------------------------------------------------------------------
		// --- Find Decisons for the "Decision" Innerwindow -------------------------
		// --------------------------------------------------------------------

		if (isset($this->_request->r)) {

			$decisions = array();
			$temp = array();


			$query = 'SELECT ?sol ?label ?description ?type WHERE { <' . $resource -> getURI() . '> <' . SWORE_SOLUTION_PROPERTY . '> ?sol' . ' . ?sol <' . EF_RDFS_LABEL . '> ?label. ?sol <http://purl.org/dc/elements/1.1/description> ?description. ?sol <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> ?t . ?t <' . EF_RDFS_LABEL . '> ?type. }';



			$qr = $this -> model -> sparqlQuery($query);
			if (is_array($qr)) {
				foreach ($qr as $f) {

					$sql =  "SELECT count(object) as Anzahl FROM statements WHERE ";
					$sql .= "predicate='http://ns.softwiki.de/req/solution' ";
					$sql .= "and subject!='http://ns.softwiki.de/req/Requirement' ";
					$sql .= "and object='".$f['sol']->getURI()."' ";
					$sql .= "and modelID IN (" . $this->model->getModelIds() . ") ";

					$sqlResult = $this->model->dbConn->Execute($sql);
					$DecCount=0;
					
					while ($row = $sqlResult->FetchRow()) {
						$DecCount=(int)$row[0];	
					};				


					$sql =  "SELECT count(object) as Anzahl FROM statements WHERE ";
					$sql .= "predicate='http://ns.softwiki.de/req/solutionFavorite' ";
					$sql .= "and subject!='http://ns.softwiki.de/req/Requirement' ";
					$sql .= "and object='".$f['sol']->getURI()."' ";
					$sql .= "and modelID IN (" . $this->model->getModelIds() . ") ";

					$sqlResult = $this->model->dbConn->Execute($sql);
					
					while ($row = $sqlResult->FetchRow()) {
						$DecCount=$DecCount+(int)$row[0];	
					};				

					$sql =  "SELECT count(object) as Anzahl FROM statements WHERE ";
					$sql .= "predicate='http://ns.softwiki.de/req/solutionExcluded' ";
					$sql .= "and subject!='http://ns.softwiki.de/req/Requirement' ";
					$sql .= "and object='".$f['sol']->getURI()."' ";
					$sql .= "and modelID IN (" . $this->model->getModelIds() . ") ";

					$sqlResult = $this->model->dbConn->Execute($sql);
					
					while ($row = $sqlResult->FetchRow()) {
						if ((int)$row[0]>0) $DecCount=0;	
					};				

					$DecFavorite='';

					$sql =  "SELECT count(object) as Anzahl FROM statements WHERE ";
					$sql .= "predicate='http://ns.softwiki.de/req/solutionFavorite' ";
					$sql .= "and subject='".$resource -> getURI()."' ";
					$sql .= "and object='".$f['sol']->getURI()."' ";
					$sql .= "and modelID IN (" . $this->model->getModelIds() . ") ";

					$sqlResult = $this->model->dbConn->Execute($sql);
					
					while ($row = $sqlResult->FetchRow()) {
						if ((int)$row[0]>0) $DecFavorite='top';	
					};				


					$sql =  "SELECT count(object) as Anzahl FROM statements WHERE ";
					$sql .= "predicate='http://ns.softwiki.de/req/solutionExcluded' ";
					$sql .= "and subject='".$resource -> getURI()."' ";
					$sql .= "and object='".$f['sol']->getURI()."' ";
					$sql .= "and modelID IN (" . $this->model->getModelIds() . ") ";

					$sqlResult = $this->model->dbConn->Execute($sql);
					
					while ($row = $sqlResult->FetchRow()) {
						if ((int)$row[0]>0) $DecFavorite='down';	
					};				



					$temp = array(
    					'uri' => $f['sol']->getURI(),
    					'label' => $f['label']->getLabel(),
   					'description' => $f['description']->getLabel(),
  					'type' => $f['type']->getLabel(), 
  					'count' => $DecCount,
					'favorite' => $DecFavorite);
					$decisions[] = $temp;
				}
			}
			$this -> view -> decisions = $decisions;
		}



		// --------------------------------------------------------------------
		// --- Set the title for the window -----------------------------------
		// --------------------------------------------------------------------

		$topicProp = $this->model->resourceF('http://www.w3.org/2004/02/skos/core#primarySubject', false);

		if (!isset($this->_request->new)) {
			$topics = $resource ? $this->model->findNodes($resource, $topicProp, null) : array();
			$topics = array_values($topics);
		}

		// all topics in use
		$topicClass = $this->model->resourceF('http://www.w3.org/2004/02/skos/core#Concept', false);
		$allTopics  = $this->model->findNodes(null, EF_RDF_TYPE, $topicClass);

		// inject them into the view
		$this->view->allTopics = array();
		foreach ($allTopics as $topic) {
			$this->view->allTopics[] = array(
                'topicUri'   => $topic->getURI(),
                'topicTitle' => $topic->getTitle($lang)
			);
		}

		// set the title for the window
		$windowTitle = '';
		if ($resource) {
			$superElementPath = array();
			if (count($topics) > 0) {
				$topics[0]->getSuperElementPath('http://www.w3.org/2004/02/skos/core#broader', $superElementPath);
			}

			// reverse the array, for the title starts with the topmost element
			$reverseSuperElementPath = array_reverse($superElementPath);

			foreach ($reverseSuperElementPath  as $element) {
				$windowTitle    .= '<a class="resource-select" about="' . $element->getUri() . '">' . htmlentities($element->getTitle($lang), ENT_NOQUOTES, 'UTF-8') . '</a>' . '&nbsp;&gt; ';
			}

			$windowTitle .= $resource->getTitle($lang);
		} else {
			$windowTitle .= $this->_strings->sw->newreqbutton; //$this->strings->newRequirement;
		}

		// --------------------------------------------------------------------
		// --- Next thing to do: Get the history informations -----------------
		// --------------------------------------------------------------------
		$this->view->allMods = array();
		if ($resource) {
			$versEngine = new Erfurt_Versioning($this->model);

			// needed to recognize localname from user-uris because
			// an Object loaded with $this->model->resourceF($someuseruri) has no information
			// in requirements- (current-)model
			
			$sysontModel = Zend_Registry::get('sysontModel');
			
			// This was used to get the Creator from the OntoWiki-versioning Engine
			// $this->view->firstMod   = $versEngine->getFirstModifiedForResource($resource);


			// get the Creator from Model
			$CreateProp = $this->model->resourceF('http://ns.softwiki.de/req/isCreatedBy', false);
			$Create = $resource ? $resource->getPropertyValue($CreateProp) : null;
			$CreatorName='unavailable';

			if (isset($Create)) {
					$CreatorName = $Create->getTitle($lang);
			} else {
				// Newer SWORE Version uses dc: creator
				$CreateProp = $this->model->resourceF('http://purl.org/dc/elements/1.1/creator', false);
				$Create = $resource ? $resource->getPropertyValue($CreateProp) : null;
				if (isset($Create)) $CreatorName = $Create->getTitle($lang);
			};

			//Add Creator of the Resource for use in JavaScript
			$this->view->Creator = $CreatorName;


			//Add User of the Resource for use in JavaScript
			$user1    = Zend_Auth::getInstance()->getIdentity();
			$this->view->userUri = $user1['uri'];


			// get the creation time from Model
			$CreateProp = $this->model->resourceF('http://purl.org/dc/terms/created', false);
			$Create = $resource ? $resource->getPropertyValue($CreateProp) : null;
			$CreationTime = ($Create instanceof Literal) ? $Create->getLabel() : '2009-04-01T00:00:00';

						
			$this->view->firstMod = array(
				'user'      =>  $CreatorName,
				'date'        => $CreationTime,
				'description' => 'Creation of this Requirement'
			);

			// Get the most recent Author
			if ($this->view->firstMod !== false) {
				$this->view->firstMod['user'] = $sysontModel->resourceF($this->view->firstMod['user'],
				false)->getTitle($lang);

				// This was used to get the latest Modifification from the OntoWiki-versioning Engine
				//$this->view->lastMod    = $versEngine->getLastModifiedForResource($resource);
				
				// get the last Editor from Model
				$LastMProp = $this->model->resourceF('http://ns.softwiki.de/req/isLastModifiedBy', false);
				$LastM = $resource ? $resource->getPropertyValue($LastMProp) : null;
				$LastMName='unavailable';
								
				if (isset($LastM)) {
					$LastMName = $LastM->getTitle($lang);
				} else {
					// Newer SWORE Version uses dc: modifier
					$LastMProp = $this->model->resourceF('http://purl.org/dc/elements/1.1/modified', false);
					$LastM = $resource ? $resource->getPropertyValue($LastMProp) : null;
					if (isset($LastM)) $LastMName = $LastM->getTitle($lang);
				};
				

				// get the modification time from Model
				$LastMProp = $this->model->resourceF('http://purl.org/dc/terms/modified', false);
				$LastM = $resource ? $resource->getPropertyValue($LastMProp) : null;
				$LastMTime = ($LastM instanceof Literal) ? $LastM->getLabel() : '2009-04-01T00:00:00';
				
				if ($LastMTime=='2009-04-01T00:00:00') {
					$LastMProp = $this->model->resourceF('http://ns.softwiki.de/req/modified', false);
					$LastM = $resource ? $resource->getPropertyValue($LastMProp) : null;
				}
				$LastMTime = ($LastM instanceof Literal) ? $LastM->getLabel() : '2009-04-01T00:00:00';

				$this->view->lastMod = array(
					'user'      =>  $LastMName,
					'date'        => $LastMTime,
					'description' => 'Latest Modification of this Requirement'
				);
				
				$this->view->lastMod['user'] = $sysontModel->resourceF($this->view->lastMod['user'], false)->getTitle($lang);


				// check whether first author = last author
				if (($this->view->lastMod['user'] === $this->view->firstMod['user'])
				&& ($this->view->lastMod['date'] === $this->view->firstMod['date'])) {

					$this->view->lastMod = false;
				} else {
					// nice up the date format
					$this->view->lastMod['date'] = Erfurt_Util::dateDifference($this->view->lastMod['date'], false, 3);
				}

				// nice up the date format
				$this->view->firstMod['date'] = Erfurt_Util::dateDifference($this->view->firstMod['date'], false, 3);



				// This was used to get all Mofications from the Ontowiki-Store
				/*
				$allMods = $versEngine->getFullHistoryForResource($resource);
				foreach ($allMods as $mod) {
					$m = $sysontModel->resourceF($mod['user'], false)->getTitle($lang);
					if (!in_array($m, $this->view->allMods)) {
						$this->view->allMods[] = $m;
					}
				}
				*/

				// get the all User related to req:isDefinedBy
				$DefByProp = $this->model->resourceF('http://ns.softwiki.de/req/isDefinedBy', false);
				$DefBys = $resource ? $resource->listPropertyValues($DefByProp) : array();
				$allMods=array();
				foreach ($DefBys as $d) {
					$m=  $d->getTitle($lang);
					$this->view->allMods[] = $sysontModel->resourceF($m, false)->getTitle($lang);
				}

			}
		}

		// --------------------------------------------------------------------
		// --- Fetch the infos about the requirement and add it ---------------
		// --------------------------------------------------------------------

		// resource title
		$rtProp = $this->model->resourceF('http://purl.org/dc/elements/1.1/title', false);
		$rt = $resource ? $resource->getPropertyValue($rtProp) : null;
		$rtValue = ($rt instanceof Literal) ? $rt->getLabel() : '';
		$this->view->resourceTitle = array(
            'titleUrl'      => OntoWiki_Util::getURL($rtProp, $this->model, 'view'),
            'titleLabel'    => $rtProp->getTitle($lang),
            'value'         => $rtValue
		);


		// get the projects

		$projectsProp = $this->model->resourceF('http://ns.softwiki.de/req/isRequirementOf', false);
		$projects = $resource ? $resource->listPropertyValues($projectsProp) : array();

		$projectsTitle = array();
		foreach ($projects as $t) {
			$projectsTitle[] = $t->getTitle($lang);
		}


		// get the tags
		$tagProp = $this->model->resourceF('http://www.holygoat.co.uk/owl/redwood/0.1/tags/taggedWithTag', false);
		$tags = $resource ? $resource->listPropertyValues($tagProp) : array();
		$tagsAsStrings = array();
		$tagsClasses = array();
		$tagsTooltips = array();
		foreach ($tags as $t) {
			$tagsAsStrings[] = $t->getTitle($lang);
			$tagResource = $this->model->instanceF($t->getQualifiedName(), false);
			$tagComment = $tagResource->getLiteralPropertyValue(EF_RDFS_COMMENT)->label;
			$tagsTooltips[] = $tagComment;
			if (strlen($tagComment)>0){
				$tagsClasses[]="tagGreen";
			}
			else $tagsClasses[]="tagBlue";
		}
		
		$sql = 'SELECT s1.object as tagUri, s2.object as tagName
					FROM statements s1, statements s2
					WHERE s1.predicate = "http://www.holygoat.co.uk/owl/redwood/0.1/tags/taggedWithTag"
					AND s2.predicate = "http://www.holygoat.co.uk/owl/redwood/0.1/tags/name"
					AND s2.object_is = "l"
					AND s1.object = s2.subject
					AND s1.modelID IN (' . $this->model->getModelIds() . ')
					GROUP BY s1.object
					LIMIT 50;
			';
		
		// get ado result set to be used with cursor
		$sqlResult = $this->model->dbConn->Execute($sql);
				
		$glossaryTags = array();
		while ($tagData = $sqlResult->FetchRow()) {
			$tagResource = $this->model->instanceF($tagData[0], false);
			$tagComment = $tagResource->getLiteralPropertyValue(EF_RDFS_COMMENT)->label;
			if (strlen($tagComment)>0){
				$glossaryTags[$tagData[1]]=$tagComment;
			}
		}
		
		//get javascript for glossary tags
		$gtags='var glossaryTags = new Array();' . PHP_EOL;
		$k=0;
		foreach ($glossaryTags as $tagName=>$glos){
			$tagName=htmlentities($tagName,ENT_COMPAT,'UTF-8');
			// change linebreaks to be compatible with javascript linebreaks
			$glos = str_replace("\n",'\\n',$glos);
			$gtags.='glossaryTags['.$k.']=new Object();' . PHP_EOL;
			$gtags.='glossaryTags['.$k.']["tagName"]="'.$tagName.'";' . PHP_EOL;
			$gtags.='glossaryTags['.$k.']["tagComment"]="'.$glos.'";' . PHP_EOL;
			// add the Glosary entry with capital letter
			$k++;
			$uc_tagName=ucfirst($tagName);
			if ($tagName!=$uc_tagName) {
				$gtags.='glossaryTags['.$k.']=new Object();'. PHP_EOL;
				$gtags.='glossaryTags['.$k.']["tagName"]="'.$uc_tagName.'";' . PHP_EOL;
				$gtags.='glossaryTags['.$k.']["tagComment"]="'.$glos.'";' . PHP_EOL;
				$k++;
			}
		}
		$this->view->glossaryTags=$gtags;
		
		//get Base URL
		$this->view->baseUrl=$this -> _request -> getBaseUrl();

		//overwrite $tagsString only if not new reqirement
		//if (!isset($this->_request->new)){
		//	$tagsString = join(', ', $tagsAsStrings);
		//}
		
		$strings = Zend_Registry::get('strings');
				
		$this->view->resourceTags = array(
            'titleUrl'      =>  OntoWiki_Util::getURL($tagProp, $this->model, 'view'),
            'titleLabel'    => $strings->sw->tags/*$tagProp->getTitle($lang)*/,
            'value'         => $tagsAsStrings,
			'classes'		=> $tagsClasses,
			'tooltips'		=> $tagsTooltips
		);
		
		// resource description
		$rdProp = $this->model->resourceF('http://purl.org/dc/elements/1.1/description', false);
		$rd = $resource ? $resource->getPropertyValue($rdProp) : null;
		$rdValue = ($rd instanceof Literal) ? $rd->getLabel() : '';
		$firstChar="A";

		foreach ($glossaryTags as $tagName => $glos){
			$tagName=htmlentities($tagName,ENT_COMPAT,'UTF-8');
			$firstChar=$tagName[0];
			if (strpos($rdValue,$tagName)){
				$rdValue=preg_replace('/(\W)'.$tagName.'(\W)/','$1<font title="'.$glos.'" style="color:rgb(0, 178, 82);">'.$tagName.'</font>$2',$rdValue);
			}
			// add the Glosary entry with capital letter
			$n_firstChar=strtoupper($tagName[0]);
			if ($n_firstChar[0] != $firstChar[0]) {
				$tagName[0]=$n_firstChar[0];
				if (strpos($rdValue,$tagName)){
					$rdValue=preg_replace('/(\W)'.$tagName.'(\W)/','$1<font title="'.$glos.'" style="color:rgb(0, 178, 82);">'.$tagName.'</font>$2',$rdValue);
				}
			}
			// add the glossary for low capital letters
			$n_firstChar=strtolower($tagName[0]);
			if ($n_firstChar[0] != $firstChar[0]) {
				$tagName[0]=$n_firstChar[0];
				if (strpos($rdValue,$tagName)){
					$rdValue=preg_replace('/(\W)'.$tagName.'(\W)/','$1<font title="'.$glos.'" style="color:rgb(0, 178, 82);">'.$tagName.'</font>$2',$rdValue);
				}
			}
		}
		$this->view->resourceDescription = array(
            'titleUrl'      => OntoWiki_Util::getURL($rdProp, $this->model, 'view'),
            'titleLabel'    => $rdProp->getTitle($lang),
            'value'         => $rdValue
		);

		// document files associated with this resource (e.g. images, pdfs)
		$rDocProp = $this->model->resourceF('http://xmlns.com/foaf/0.1/isPrimaryTopicOf', false);
		$rDocs = $resource ? $resource->listPropertyValues($rDocProp) : array();

		$this->view->resourceDocuments = array();
		foreach ($rDocs as $doc) {
			if ($doc->getPropertyValue(EF_RDF_TYPE)->getURI() === 'http://xmlns.com/foaf/0.1/Image') {
				$rDocType = 'image';
			} else {
				$rDocType = 'document';
			}

			$this->view->resourceDocuments[] = array(
                'docUrl'    => $doc->getURI(),
                'docType'   => $rDocType,
                'docTitle'  => $doc->getTitle($lang)
			);
		}

		// topic poperty info are always needed
		$this->view->resourceTopic = array(
            'titleUrl'      => OntoWiki_Util::getURL($topicProp, $this->model, 'view'),
            'titleLabel'    => $topicProp->getTitle($lang)
		);
		// get the topic
		if (count($topics) > 0) {
			$this->view->resourceTopic['value']    = $topics[0]->getTitle($lang);
			$this->view->resourceTopic['valueUri'] = $topics[0]->getURI();
		}


		// Version Title

		// TR: get all Versions using SQL

		$store = Zend_Registry::get('erfurt')->getStore();

		$versionsSQL = 'SELECT DISTINCT log_s.object AS description, log_a.ef_date AS ef_date, log_a.ef_user AS ef_user, log_s.action_id AS action_id
FROM log_statements AS log_s, log_actions AS log_a
WHERE log_s.subject = "'.$resource->getUri().'"
AND log_s.action_id = log_a.id
AND log_s.predicate = "http://purl.org/dc/elements/1.1/description"
AND log_s.ar = "a"
AND (log_a.descr_id =1 OR log_a.descr_id =4)
ORDER BY log_s.action_id DESC';

		$this->view->resourceVersions = $store->sqlQuery($versionsSQL);



		// add current project id

		$store = Zend_Registry::get('erfurt')->getStore();

		$doapProjectSQL = "SELECT DISTINCT subject FROM statements WHERE predicate = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#type' AND object = 'http://usefulinc.com/ns/doap#Project' AND modelID = " . $this->model->getModelID();
		$doapProjectURI="";
		$resultData = $store->sqlQuery($doapProjectSQL);
		foreach ($resultData as $doapProject) {
			$doapProjectURI = $doapProject[0];
			break;			
		}
		$this->view->resourceTopic['project']    = $doapProjectURI;


		// get the projects

		$this->view->resourceTopic['projects'] = "";
		$this->view->resourceTopic['projectsTitle'] = "";

		$projectsProp = $this->model->resourceF('http://ns.softwiki.de/req/isRequirementOf', false);
		$projects = $resource ? $resource->listPropertyValues($projectsProp) : array();

		$projectsTitle = array();
		foreach ($projects as $t) {
			if ($this->view->resourceTopic['projects']!="") $this->view->resourceTopic['projects'] .= ", ";
 			$this->view->resourceTopic['projects'] .= $t->getURI();
			if ($this->view->resourceTopic['projectsTitle']!="") $this->view->resourceTopic['projectsTitle'] .= ", ";
			$this->view->resourceTopic['projectsTitle'] .= $t->getTitle($lang);
		}



		// --------------------------------------------------------------------
		// --- Build the JavaScript model -------------------------------------
		// --------------------------------------------------------------------

		if ($resource) {
			// start with an empty memory model
			$pageModel = new MemModel();

			// add dc:title
			$pageModel->add(
			new Statement(
			$resource,
			$rtProp,
			(($rt instanceof Literal) ? $rt : new Literal(''))
			)
			);

			// add dc:description
			$pageModel->add(
			new Statement(
			$resource,
			$rdProp,
			(($rd instanceof Literal) ? $rd : new Literal(''))
			)
			);

			// add projects titles
			foreach ($projects as $proj) {
				$pageModel->add(
				new Statement(
				$resource,
				$projectsProp,
				$proj
				)
				);
			}


			// add documents and their type and label
			foreach ($rDocs as $doc) {
				$pageModel->add(
				new Statement(
				$resource,
				$rDocProp,
				$doc
				)
				);

				// TODO catch the case that multiple rdf:type definitions exist
				$pageModel->add(
				new Statement(
				$doc,
				new Resource(EF_RDF_TYPE),
				$doc->getPropertyValue(EF_RDF_TYPE)
				)
				);

				// TODO catch the case that multiple rdfs:label definitions exist
				$pageModel->add(
				new Statement(
				$doc,
				new Resource(EF_RDFS_LABEL),
				$doc->getPropertyValue(EF_RDFS_LABEL)
				)
				);
			}

			// add topic
			if (count($topics) > 0) {
				$pageModel->add(
				new Statement(
				$resource,
				$topicProp,
				$topics[0]
				)
				);
			}

			// add tags
			foreach ($tags as $tag) {
				$pageModel->add(
				new Statement(
				$resource,
				$tagProp,
				$tag
				)
				);
			}
		}

		// append the uri for the form
		$this->view->requri = $resource ? $resource->getUri() : '';

		// render the softwiki view template and assign to content
		$this->view->addScriptPath(REAL_BASE . 'plugins/SoftWiki/templates/');
		$content = $this->view->render('softwiki/view.php');
		$this->view->clearVars();

		// render the window template with the content from here
		$this->view->title      = $windowTitle;
		$this->view->content    = $content;
		$this->view->cssClasses = 'softwiki-view';



		$this->_response->setBody($this->view->render('micro/window.php'));


		return;
	}

	/**
	 * Enter description here...
	 *
	 */
	public function listAction() {
		// set window title
		#$this->view->placeholder('title')->set('SoftWiki Main Window');

		// overwrite default sidebar
		$this->view->addScriptPath(dirname(__FILE__));

		$this->view->placeholder('sidebar')->set($this->view->partial('sidebar.php'));

		// echo bullshit
		#echo 'List action rendered by the SoftWiki plug-in. :)';
		#print ' | pfad = '.$this->config->uriBase . 'plugins/SoftWiki/templates/resource/listiger.php';
		#$this->config->uriBase;
		#$this->view->assign('content', $this->view->render($this->config->themes->path . 'resource/index.php'));


		$vr = Zend_Controller_Action_HelperBroker::getStaticHelper('viewRenderer');
		$this->view->addScriptPath(REAL_BASE.'plugins/SoftWiki/templates/');
		$vr->setScriptAction('list');
		// disable template rendering (for now)
		#$this->_helper->viewRenderer->setNoRender();
	}





	/**
	 * Generate List of Reqirements with given
	 * Parameters in GET/POST
	 */
	public function reqlistAction() {

		// as a service it needs no view renderer
		$this->_helper->viewRenderer->setNoRender();
		// as a service it needs no layout template
		$this->_helper->layout->disableLayout();

		// get all needed parameters from GET and strip slashes if needed
		// stripslashes for all values
		if (get_magic_quotes_gpc()) {
			foreach ($_GET as $key => $value) {
				//+ in params gets a space, have to put that back
				$params[$key] = str_replace(' ','+',stripslashes($value));
			}
		} else {
			$params = $_GET;
		}
		
		//get params into view for back function
		$this->view->backParams=$params;

		// initialize the tags and topics arrays
		$tags = array();
		$topics = array();

		$session = Zend_Registry::get('session');
		$lang = $session->modelLanguage;
		$listdata = array();

		// for saving uris of already found resources
		$foundResources = array();

		if (isset($params['advancedConfig']) && $params['advancedConfig'] == "true") {
			
			$this->view->showRelated = true;
			if (isset($params['r'])) {
				$foundResources[] = $params['r'];
			}
			
			if (isset($params['relations']) &&  $params['relations'] == "true") {
				$checkRelated = 'checked="true"';
			} else {
				$checkRelated = '';
			}

			if (isset($params['similar']) && $params['similar'] != '') {
				$checkSimilar = 'checked="true"';
			} else {
				$checkSimilar = '';
			}
				
			if (isset($params['all']) && $params['all'] == 'true') {
				$checkAll = 'checked="true"';
			} else {
				$checkAll = '';
			}
			
			$this->view->configBlock = '<div class="float-left" style="margin:0.5em" title="' .
				$this->strings->sw->tooltip->reqlist->showrelated . '">' . PHP_EOL .
				$this->strings->sw->reqlist->showrelated . PHP_EOL .
				'<input type="checkbox" ' . $checkRelated . ' class="toggleReqRelatedCheckbox"/></div>';
			$this->view->configBlock .= '<div class="float-left" style="margin:0.5em" title="' .
				$this->strings->sw->tooltip->reqlist->showsimilar . '">' . PHP_EOL .
				$this->strings->sw->reqlist->showsimilar . PHP_EOL .
				'<input type="checkbox" ' . $checkSimilar . ' class="toggleReqSimilarCheckbox"/></div>';
			$this->view->configBlock .= '<div class="float-left" style="margin:0.5em" title="' .
				$this->strings->sw->tooltip->reqlist->showundefined . '">' . PHP_EOL .
				$this->strings->sw->reqlist->showundefined . PHP_EOL .
				'<input type="checkbox" ' . $checkAll . ' class="toggleReqUnrelatedCheckbox"/></div>';
		}


		// Look for related requirements
		if (isset($params['relations']) && isset($params['r']) && $params['relations'] == "true") {
			$this->view->showRelated = true;

			$res = $this->model->sparqlQuery('SELECT DISTINCT * WHERE {<' . $params['r'] . '> ' .
			    '?p ?y . ?y a <' . SWORE_REQUIREMENT . '> . }');

			foreach ($res as $row) {

				$relation = $this->model->resourceF($row['p']);
				$description = $row['y']->getPropertyValue('dc:description');

				if (null !== $description) {
					$description_short  = substr($description->getLabel(), 0, 60) . '...';
					$description_full   = $description->getLabel();
				} else {
					$description_short  = '';
					$description_short  = '';
				}

				if ( !in_array($row['y']->getURI(), $foundResources) && $row['p']->getURI() != EF_RDF_TYPE) {
					$foundResources[] = $row['y']->getURI();
					$listdata[] = array(
                        'resourceUri'           => $row['y']->getURI(),
                        'resourceLabel'         => $row['y']->getTitle(),
                        'resourceComment_short' => $description_short,
                        'resourceComment_full'  => $description_full,
                        'reqId'                 => $this->_getReqId($row['y']->getURI()),
				        'reqRelated'            => $relation->getTitle($lang)
					);
				} else {
					// do nothing
				}



			}

		}

		// if the similar parameter is given we want to use the textmining plugin
		if (isset($params['similar'])) {
			$tmPlugin = Zend_Registry::get('erfurt')->getPluginManager()->prepare('Textmining');

			if ($tmPlugin) {
				$result = $tmPlugin->findSimilarInstances($params['similar']);
			} else {
				$result = array();
			}





			foreach ($result as $row) {
				if ($row === $params['r']) {
					continue;
				}
				$r = $this->model->resourceF($row, false);
				
				// Adding uri not to be found multiple times
				if (in_array($r->getURI(),$foundResources)) {
					continue;
				} else {
					$foundResources[] = $r->getURI();
				}
				$label = $r->getTitle();
				$description = $r->getPropertyValue('dc:description');

				if (null !== $description) {
					$description_short  = substr($description->getLabel(), 0, 60) . '...';
					$description_full   = $description->getLabel();
				} else {
					$description_short  = '';
					$description_short  = '';
				}
				
				$listdata[] = array(
                    'resourceUri'           => $r->getURI(),
                    'resourceLabel'         => $label,
                    'resourceComment_short' => $description_short,
                    'resourceComment_full'  => $description_full,
                    'reqId'                 => $this->_getReqId($r->getURI())
				);
			}
		}
		if (isset($params['all']) && $params['all'] == 'true') {

			// fetch the topic and tag uris from params
			if (!isset($params['resources'])) {
				$uris = array();
			} else {
				$uris = json_decode($params['resources']);

				if (!is_array($uris)) {
					//throw new Erfurt_Exception('json decode error.');
				}
			}

			// throw away SWORE_REQUIREMENT uri
			$newUris = array();
			foreach($uris as $uri) {
				if (!($uri === SWORE_REQUIREMENT)) {
					$newUris[] = $uri;
				}
			}
			$uris = $newUris;

			// get store and model id
			$store = Zend_Registry::get('erfurt')->getStore();
			$modelID = $this->model->getModelID();

			foreach ($uris as $u) {
				$r = $this->model->resourceF($u, false);
				$type = $r->getPropertyValue(EF_RDF_TYPE);

				if (($type instanceof Erfurt_Rdfs_Resource)
				&& ($type->getURI() === 'http://www.w3.org/2004/02/skos/core#Concept')) {

					$topics[] = $r->getURI();
				} else {
					$tags[] = $r->getURI();
				}
			}


			// build the transitive closure for the topics
			$transTopics = $topics;
			$hasChanged = false;
			do {
				$hasChanged = false;
				foreach ($transTopics as $u) {
					if (!$u instanceof Erfurt_Rdfs_Resource) {
						$u = $this->model->resourceF($u, false);
					}
					$list = $u->listPropertyValuesObject('http://www.w3.org/2004/02/skos/core#broader');

					if ($list) {
						foreach($list as $l) {
							if (!in_array($l->getURI(), $transTopics)) {
								$transTopics[] = $l->getURI();
								$hasChanged = true;
							}
						}
					}
				}
			} while ($hasChanged);


			if (!isset($params['search'])) {
				$search = '';
			} else {
				if (strlen($params['search']) < 3 ) {
					$search = '';
				} else {
					$search = $params['search'];
				}
			}

			if (!isset($params['limit'])) {
				$limit = $session->count;
			} else {
				$limit = $params['limit'];
			}

			if (!isset($params['offset'])) {
				$offset = 0;
			} else {
				$offset = $params['offset'];
			}

			// Doing data operations
			// no uris (topics and tags) given means
			if (sizeof($uris) === 0 ) {
/*				$queryNoLimit ='SELECT s2.subject FROM statements s '.
							'LEFT JOIN statements s2 ON (s.modelID = s2.modelID AND s.subject = s2.subject) '.
							'WHERE s.modelID = '.$modelID.' AND s2.predicate = "' . EF_RDF_TYPE .'" AND s2.object = "' .
				SWORE_REQUIREMENT .'" AND s.predicate = "http://purl.org/dc/elements/1.1/title" '.
							'ORDER by s.object';
*/
				$queryNoLimit ='SELECT DISTINCT subject FROM statements '.
							'WHERE modelID = '.$modelID.' AND predicate = "'.SWORE_ISREQUIREMENTOF.'" ';

				$query = $queryNoLimit;

				$res = $store->sqlQuery($query);

				foreach ($res as $c) {
					$resultset[$c[0]] = $this->model->resourceF($c[0]);

					if (!in_array($resultset[$c[0]]->getUri(),$foundResources)) {
						$foundResources[] = $resultset[$c[0]]->getUri();
						$temparray = array();
						$temparray['resourceUri'] = $resultset[$c[0]]->getUri();
						$temparray['resourceLabel']=  $resultset[$c[0]]->getTitle();
						if ($resultset[$c[0]]->hasPropertyValue('dc:description') ) {
							$temparray['resourceComment_short'] =
							substr($resultset[$c[0]]->getPropertyValue('dc:description')->getLabel(),0,60) . '...';
							$temparray['resourceComment_full'] =
							$resultset[$c[0]]->getPropertyValue('dc:description')->getLabel();
						} else {
							$temparray['resourceComment_short'] = '';
							$temparray['resourceComment_full'] = '';
						}
						$temparray['reqId'] = $this->_getReqId($resultset[$c[0]]->getURI());
						
						$doapProjectTitle="";

						$tempProjects = $resultset[$c[0]]->listPropertyValues('req:isRequirementOf');
						foreach ($tempProjects as $tempP) {
							if ($doapProjectTitle!="") $doapProjectTitle .= ", "; 								$doapProjectTitle .= $tempP->getTitle($lang) ;
						}

						//TR-BMW
						$temparray['doapProject']=$doapProjectTitle;

						$listdata[] = $temparray;


					} else {
						// do nothing
					}
				}
	
			} else {
				$res = array();

				// $queryNoLimit = 'SELECT s2.subject FROM statements s
				//                      LEFT JOIN statements s2 ON (s.modelID = s2.modelID AND s.subject = s2.subject)
				//                      WHERE s.modelID = ' . $modelID . ' AND s2.predicate = "' . EF_RDF_TYPE .'"
				//                      AND s2.object = "' . SWORE_REQUIREMENT .'"
				//                      AND s.predicate = "http://purl.org/dc/elements/1.1/title"
				//                      AND ' . count($tags) . ' = (
				//                          SELECT count(s3.subject) FROM statements s3 WHERE
				//                          s3.subject = s2.subject AND s3.subject_is = s2.subject_is AND
				//                          s3.predicate = "http://www.holygoat.co.uk/owl/redwood/0.1/tags/taggedWithTag" AND
				//                          s3.object_is = "r" AND s3.object IN
				//                          ("' . join('", "', $tags) . '")
				//                      )';
/*
				$queryNoLimit = 'SELECT s.subject FROM statements s '.
						 		'LEFT JOIN statements s2 ON (s.modelID = s2.modelID AND s.subject = s2.subject '.
						 		'AND s2.predicate = "http://www.w3.org/2004/02/skos/core#primarySubject" '.
						 		'AND s2.object IN ("' . join('", "', $transTopics) . '")) '.
						 		'LEFT JOIN statements s3 ON (s.modelID = s3.modelID AND s.subject = s3.subject '.
						 		'AND s3.predicate = "http://purl.org/dc/elements/1.1/title" ) '.
						 		'WHERE s.modelID = ' . $modelID . ' AND s.predicate = "' . EF_RDF_TYPE .'" AND s.object = "' . SWORE_REQUIREMENT .'" '. 

						 		'AND ' . count($tags) . ' = (
                    			    SELECT count(s4.subject) FROM statements s4 WHERE s4.modelID = ' . $modelID . ' AND ' .
                    			    's4.subject = s.subject AND s4.subject_is = s.subject_is AND
                    			    s4.predicate = "http://www.holygoat.co.uk/owl/redwood/0.1/tags/taggedWithTag" AND
                    			    s4.object_is = "r" AND s4.object IN
                    			    ("' . join('", "', $tags) . '") 
                    			)';
*/
				$queryNoLimit = 'SELECT s.subject FROM statements s '.
						 		'LEFT JOIN statements s2 ON ( s.subject = s2.subject '.
						 		'AND s2.predicate = "http://www.w3.org/2004/02/skos/core#primarySubject" '.
						 		'AND s2.object IN ("' . join('", "', $transTopics) . '")) '.
						 		'LEFT JOIN statements s3 ON (s.modelID = s3.modelID AND s.subject = s3.subject '.
						 		'AND s3.predicate = "http://purl.org/dc/elements/1.1/title" ) '.
						 		'WHERE s.modelID = ' . $modelID . ' AND '.
								       's.predicate = "'.SWORE_ISREQUIREMENTOF.'" '. 
						 		'AND ' . count($tags) . ' = (
                    			    SELECT count(s4.subject) FROM statements s4 WHERE ' .
                    			    's4.subject = s.subject AND s4.subject_is = s.subject_is AND
                    			    s4.predicate = "http://www.holygoat.co.uk/owl/redwood/0.1/tags/taggedWithTag" AND
                    			    s4.object_is = "r" AND s4.object IN
                    			    ("' . join('", "', $tags) . '") 
                    			)';

				//echo $queryNoLimit;

				if (count($transTopics) > 0) {
					$queryNoLimit .= ' AND s2.predicate IS NOT NULL';
				}

				//$query= $queryNoLimit . ' ORDER by s3.object LIMIT ' .$limit . ' OFFSET ' . $offset;
				$query = $queryNoLimit . ' ORDER BY s3.object';


				$res = $store->sqlQuery($query);

				//				// count for the pager
				//				$countResult = $store->sqlQuery(str_replace('SELECT s.subject', 'SELECT COUNT(s.subject)', $queryNoLimit));
				//				$count = (int) $countResult[0][0];
				# testausgaben
				#                echo $query;
				#                var_dump($res);exit;
				# ende test

				foreach ($res as $c) {
					$r = $this->model->resourceF($c[0], false);
					#$tempTags = $r->listPropertyValues('http://www.holygoat.co.uk/owl/redwood/0.1/tags/taggedWithTag');
					#$usedTags = array();
					#foreach ($tempTags as $temp) {
					#    $usedTags[] = $temp->getURI();
					#}

					#$valid = true;
					#foreach ($tags as $tag) {
					#    if (!in_array($tag, $usedTags)) {
					#        $valid = false;
					#        break;
					#    }
					#}
					#if (!$valid) {
					#    continue;
						#}

						$resultset[$c[0]] = $r;
						if (!in_array($resultset[$c[0]]->getUri(),$foundResources)) {
							$foundResources[] = $resultset[$c[0]]->getUri();
							if ($dcDescription = $resultset[$c[0]]->getPropertyValue('dc:description')) {
								$full  = $dcDescription->getlabel();
								$short = substr($full, 0, 60) . '&hellip;';
							}
							$tempProjects = $resultset[$c[0]]->listPropertyValues('req:isRequirementOf');
							$doapProjectTitle="";
							foreach ($tempProjects as $tempP) {
								if ($doapProjectTitle!="") $doapProjectTitle .= ", "; 								$doapProjectTitle .= $tempP->getTitle($lang) ;
							}
							//TR-BMW


							if ($doapProject = $resultset[$c[0]]->getPropertyValue('req:isRequirementOf')) {
								$doapProjectTitle=$doapProject->getTitle($lang) ;
							} else  { $doapProjectTitle="keine Projekt angegeben"; } //TR-BMW
							
							$listdata[$resultset[$c[0]]->getTitle($lang)] = array(
                        	'resourceUri'           => $resultset[$c[0]]->getUri(),
                        	'resourceLabel'         => $resultset[$c[0]]->getTitle($lang),
                        	'resourceComment_short' => $short,
                        	'resourceComment_full'  => $full,
                        	'reqId'                 => $this->_getReqId($resultset[$c[0]]->getURI()),
                        	'doapProject'           => $doapProjectTitle  //TR-BMW
							);
						} else {
							// do nothing
						}
					}
			}


			// --------------------------------------------------------------------
			// --- sortierung des ergebnis arrays ---------------------------------
			// --------------------------------------------------------------------

			/*      #ksort($listdata);
			 // Hole eine Liste von Spalten
			 foreach ($listdata as $key => $row) {
			 $resourceLabel[$key]  = $row['resourceLabel'];
			 }
			 // Die Daten mit resourceLabel absteigend sortieren
			 array_multisort($resourceLabel, SORT_ASC, $listdata);
			 */


			if ($search != '') {
				foreach ($listdata  as $key => $array) {
					if (($pos = stripos($array['resourceLabel'], $search)) !== false) {
						$foundString = substr($array['resourceLabel'], $pos, strlen($search));
						$array['resourceLabel'] = str_ireplace($search,'<span class="highlighted">' . $foundString . '</span>',$array['resourceLabel']);
						$listdata[$key] = $array;
					} else if (($pos = stripos($array['resourceComment_full'], $search)) !== false) {
						$foundString = substr($array['resourceComment_full'], $pos, strlen($search));
						$array['resourceComment_full'] = str_ireplace($search,'<span class="highlighted">' . $foundString . '</span>',$array['resourceComment_full']);
						$array['resourceComment_short'] = str_ireplace($search,'<span class="highlighted">' . $foundString . '</span>',$array['resourceComment_short']);
						$listdata[$key] = $array;
					} else {
						unset($listdata[$key]);
					}
				}

			}

			// count for pager
			$count = count($listdata);
				
			if ($count > $limit) {
				$listdata = array_slice($listdata, $offset, $limit);
			}
				
		}
		// Setting up the view variables for the reqlist template
		$quicksearchparams = array(
            'uris' => json_encode($uris),
            'search' => $search,
            'similar' => $params['similar']
		);

		$this->view->quicksearchparams = json_encode($quicksearchparams);
		$this->view->similar = $params['similar'];
		$this->view->search = $search;
		$this->view->listdata = $listdata;
		$this->view->baseUrl = $this->config->uriBase;
        $this->view->modelUri = $this->model->getModelURI();

		// pager vars
		$this->view->pagerLinks = $this->_getPagerLinks($count, $limit, $offset);
		$this->view->count      = $count;
		$this->view->limit      = $limit;
		$this->view->offset     = $offset;
		$this->view->pageUrl    = $this->_request->getServer('QUERY_STRING');

		// Add Script Path to find templates needed in following 'render'-methods
		$this->view->addScriptPath(REAL_BASE.'plugins/SoftWiki/templates/');

		// Render Content of a window
		$content = $this->view->render('softwiki/reqlist.php');

		// Clearing all previous view variables
		$this->view->clearVars();

		// set up the window title
		if (isset($params['similar'])) {
			$windowTitle = $this->strings->sw->reqview->title->similarrequirements;
		} else if (isset($params['relations']) && $params['relations'] == "true") {
			$windowTitle = $this->strings->sw->reqlist->listrelated;
		} else {
			if (count($topics) === 0) {
				$windowTitle = $this->strings->sw->reqlist->listall;
			} else {
				$windowTitle = '';
			}

			$topicsAsString = array();
			foreach ($topics as $t) {
				$t = $this->model->resourceF($t, false);
				$topicsAsString[] = $t->getTitle($lang);
			}
			$tagsAsString = array();
			foreach ($tags as $t) {
				$t = $this->model->resourceF($t, false);
				$tagsAsString[] = $t->getTitle($lang);
			}
			$windowTitle .= join(', ', $topicsAsString);
			if (count($tags) > 0) {
				$windowTitle .= ' + ';
			}
			$windowTitle .= join(', ', $tagsAsString);
			if ($search !== '') {
				$windowTitle .= ' (Search Results)';
			}
		}


		// Setting up variables for micro/window.php template
		$this->view->title = $windowTitle;
		$this->view->content = $content;
		$this->view->cssClasses = 'softwiki-list hidden-overflow';

		$this->_response->setBody($this->view->render('micro/window.php'));
	}

	/**
	 * Enter description here...
	 *
	 * @param unknown_type $count
	 * @param unknown_type $limit
	 * @param unknown_type $offset
	 * @return unknown
	 */
	private function _getPagerLinks($count, $limit, $offset) {
		if ($count <= $limit) {
			return array();
		}

		$firstHtml = '&laquo;&nbsp;'.$this->strings->sw->reqlist->pager->first;
		$prevHtml  = '&lsaquo;&nbsp;'.$this->strings->sw->reqlist->pager->previous;
		$nextHtml  = $this->strings->sw->reqlist->pager->next .'&nbsp;&rsaquo;';
		$lastHtml  = $this->strings->sw->reqlist->pager->last .'&nbsp;&raquo;';

		$ret = array();

		if ($count > $limit) {
			if ($offset > 0) {
				$ret[] = '<a class="pager first-page">' . $firstHtml . '</a>';
				$ret[] = '<a class="pager previous-page">' . $prevHtml . '</a>';
			} else {
				$ret[] = '<a class="disabled">' . $firstHtml . '</a>';
				$ret[] = '<a class="disabled">' . $prevHtml . '</a>';
			}
		}

		for ($i = max($offset / $limit - 3, 0); $i < min($offset / $limit + 4, $count / $limit); $i++) {
			if ($offset == $i * $limit) {
				$ret[] = '<a class="pager page selected" id="page-' . ($i + 1) . '">' . ($i + 1) . '</a> ';
			} else {
				$ret[] = '<a class="pager page" id="page-' . ($i + 1) . '">' . ($i + 1) . '</a>';
			}
		}

		if ($offset + $limit < $count) {
			$ret[] = '<a class="pager next-page">' . $nextHtml . '</a>';
			$ret[] = '<a class="pager last-page">' . $lastHtml . '</a>';
		} else {
			$ret[] = '<a class="disabled">' . $nextHtml . '</a>';
			$ret[] = '<a class="disabled">' . $lastHtml . '</a>';
		}

		return $ret;
	}

	/**
	 * Enter description here...
	 *
	 */
	public function updateAction() {
		/*
		 * example:
		 * rtitle=Sumere+medicus&
		 * rdescription=Itrasci+passus+acer+extra+medicus+ideo+sunt%2C+ventus&
		 * rtopic=Geheimhaltung&
		 * rtags=Frage%2C+l%C3%A4uft%2C+Spa%C3%9F%2C+Team&r=http://testdb.softwiki.de/Req12
		 * r=http://testdb.softwiki.de/Req12
		 */

		if (!$this->model) {
			return;
		}

		$store = $this->model->getStore();

		$TITLE_PROPERTY       = 'http://purl.org/dc/elements/1.1/title';
		$DESCRIPTION_PROPERTY = 'http://purl.org/dc/elements/1.1/description';
		$TOPIC_PROPERTY       = 'http://www.w3.org/2004/02/skos/core#primarySubject';
		$TAG_PROPERTY         = 'http://www.holygoat.co.uk/owl/redwood/0.1/tags/taggedWithTag';
		$TAG_NAME_PROPERTY    = 'http://www.holygoat.co.uk/owl/redwood/0.1/tags/name';
		$CREATED_BY_PROPERTY  = 'http://ns.softwiki.de/req/isCreatedBy';
		$DEFINED_BY_PROPERTY  = 'http://ns.softwiki.de/req/isDefinedBy';
		$LAST_MODIFIED_BY_PROPERTY  = 'http://ns.softwiki.de/req/isLastModifiedBy';
		$MODIFIED_PROPERTY  = 'http://ns.softwiki.de/req/modified';
		$CREATION_PROPERTY    = 'http://purl.org/dc/terms/created';
		$LAST_MODIFIED_PROPERTY    = 'http://purl.org/dc/terms/modified';
		
		$this->_helper->viewRenderer->setNoRender();
		$this->_helper->layout->disableLayout();

		if (isset($this->_request->rtitle)) {
			$title = $this->_request->rtitle;
			Zend_Registry::get('owLog')->debug('title: ' . $title);
		}

		if (isset($this->_request->rdescription)) {
			if (get_magic_quotes_gpc()) {
				$description =  stripslashes($this->_request->rdescription);
			} else {
				$description = $this->_request->rdescription;
			}
			$description=$description;
			Zend_Registry::get('owLog')->debug('descr: ' . $description);
		}

		if (isset($this->_request->rtopic)) {
			$topic = $this->_request->rtopic;
			Zend_Registry::get('owLog')->debug('topic: ' . $topic);
		}

		$tags = array();
		if (isset($this->_request->rtags)) {
			if (get_magic_quotes_gpc()) {
				$tags = json_decode(stripslashes($this->_request->rtags));
			} else {
				$tags = json_decode($this->_request->rtags);
			}

			Zend_Registry::get('owLog')->debug('tags: ' . serialize($tags));
		}
		
		//need that for the add statement(can't add dcterms:modified with setPropertyValues without URI before date)
		$resourceUri="";
		$addRequirement=true;
		if (isset($this->_request->r) && $this->_request->r != '') {
			$addRequirement=false;
			// requirement updated, save changes
			$req = $this->model->resourceF($this->_request->r, false);
			//remove old modified date; is not necessary after adding creation Properties 
			$removeModifiedDateSql = 'DELETE FROM statements where subject ="' . $this->_request->r . '" AND predicate = "' . $LAST_MODIFIED_PROPERTY . '"';
			$result = $store->sqlQuery($removeModifiedDateSql);
			//remove old modified User; is not necessary after adding creation Properties 
			$removeModifiedBySql = 'DELETE FROM statements where subject ="' . $this->_request->r . '" AND predicate = "' . $LAST_MODIFIED_BY_PROPERTY . '"';
			$result = $store->sqlQuery($removeModifiedBySql);
			//set resourceURI to that in request
			$resourceUri=$this->_request->r;
		} else {
			// new requirement
			$prefix = ($this->model->getBaseURI() != '' ? 'Req' : ($this->model->modelURI . 'Req'));
			$newUri = $this->model->getUniqueResourceURI($prefix);
			$req    = $this->model->resourceF($newUri, false);
			//set resourceURI to that in request
			$resourceUri=$newUri;
		}

		Zend_Registry::get('owLog')->debug('req: ' . $req->getURI());
		$user    = Zend_Auth::getInstance()->getIdentity();
		$userUri = $user['uri'];
		$date=date('Y-m-d\TH:i:s');
		

		// set property values
		$req->setPropertyValue(EF_RDF_TYPE, SWORE_REQUIREMENT);
		$req->setPropertyValue($TITLE_PROPERTY, $title);
		$req->setPropertyValue($DESCRIPTION_PROPERTY, $description);
		$req->setPropertyValue($TOPIC_PROPERTY, $topic);
		
		// get current Project
		$doapProjectSQL = "SELECT DISTINCT subject FROM statements WHERE predicate = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#type' AND object = 'http://usefulinc.com/ns/doap#Project' AND modelID = " . $this->model->getModelID();
		$doapProjectURI="";
		$resultData = $store->sqlQuery($doapProjectSQL);
		foreach ($resultData as $doapProject) {
			$doapProjectURI = $doapProject[0];
			break;			
		}
		if ($doapProjectURI!="") {

			// add current Project
			$newStatement=new Statement(new Resource($resourceUri),new Resource("http://ns.softwiki.de/req/isRequirementOf"), new Resource($doapProjectURI));
			$this->model->addStatementArray(array($newStatement));

		};
		// add Creator and date, if new Requirement added, only 		
		if ($addRequirement) {
			$req->setPropertyValue($CREATED_BY_PROPERTY, $userUri);
			$newStatement=new Statement(new Resource($resourceUri),new Resource($CREATION_PROPERTY), new Literal($date));
			$this->model->addStatementArray(array($newStatement));
		}



		// always add datetime of Requirement Modifcation
		$newStatement=new Statement(new Resource($resourceUri),new Resource($MODIFIED_PROPERTY), new Literal($date));
		$this->model->addStatementArray(array($newStatement));

		// Last Modification
		$newStatement=new Statement(new Resource($resourceUri),new Resource($LAST_MODIFIED_PROPERTY), new Literal($date));
		$this->model->addStatementArray(array($newStatement));

		// Last Modification
		$newStatement=new Statement(new Resource($resourceUri),new Resource($LAST_MODIFIED_BY_PROPERTY), new Resource($userUri));
		$this->model->addStatementArray(array($newStatement));


        // always add Stakeholder at any time of Modifikation, so far
		$newStatement=new Statement(new Resource($resourceUri),new Resource($DEFINED_BY_PROPERTY), new Resource($userUri));
		$this->model->addStatementArray(array($newStatement));
		

		
		// get tag URIs
		$tagsSql = "
		SELECT DISTINCT subject, object
		FROM statements
		WHERE predicate = '$TAG_NAME_PROPERTY'
		AND object in ('" . implode($tags, "','") . "')
		AND modelID = " . $this->model->getModelID();

		// the tags we will add to the resource
		$realTags = array();
		// tags twho already have URIs
		$existingTags = array();
		// get data for existing tags
		$tagData = $store->sqlQuery($tagsSql);
		// secure forach
		if (!$tagData) {
			$tagData = array();
		}

		// filter out tags that already have URIs yet
		foreach ($tagData as $tagRow /* tagURI, tagName */) {
			if (in_array($tagRow[1], $tags)) {
				$existingTags[] = $tagRow[1];
				$realTags[] = $tagRow[0];
			}
		}

		// diff with all tags to yield new tags
		$newTags = array_diff($tags, $existingTags);

		// create resources for new tags
		foreach ($newTags as $tag) {
			$tagUri = str_replace('ä', 'ae', $tag);
			$tagUri = str_replace('Ä', 'Ae', $tagUri);
			$tagUri = str_replace('ö', 'oe', $tagUri);
			$tagUri = str_replace('Ö', 'Oe', $tagUri);
			$tagUri = str_replace('ü', 'ue', $tagUri);
			$tagUri = str_replace('Ü', 'Ue', $tagUri);
			$tagUri = str_replace('ß', 'ss', $tagUri);
			$tagUri = str_replace(' ', '', $tagUri);

			$newTagUri = $this->model->getUniqueResourceURI(urlencode($tagUri));
			$newTagResource = $this->model->resourceF($newTagUri);

			$newTagResource->setPropertyValue($TAG_NAME_PROPERTY, $tag);
			// add to tags to be storen on resource
			$realTags[] = $newTagUri;
		}

		// remove all tags hardcode since API won't :(
		// if (($this->_request->r) && $this->_request->r != 'undefined') {
		//     $removeTagsSql = 'DELETE FROM statements where subject ="' . $this->_request->r . '" AND predicate = "' . $TAG_PROPERTY . '"';
			//     $result = $store->sqlQuery($removeTagsSql);
			// }
			//
		// if (!empty($realTags)) {
		//     $modelId = $this->model->modelID;
			//     $resource = $req->getURI();
			//     $addTagsSql = '';
			//     foreach ($realTags as $tag) {
			//         $addTagsSql = "
				//             INSERT INTO statements
				//             (modelID, subject, predicate, object, subject_is, object_is)
				//             VALUES ($modelId, '$resource', '$TAG_PROPERTY', '$tag', 'r', 'r');";
				//         $store->sqlQuery($addTagsSql);
		//     }
		// }

		// add all tags to the resource
		$req->setPropertyValues($TAG_PROPERTY, $realTags);
		
		// Doing exit not to have 404s because of unfound templates
		// should output Resource URI on creation to reload correct page
		echo 'URI:' . rawurlencode($newUri);
		exit();
}

/**
 * Render Screen to select from multiple requirement models
 * normally called after login
 */
public function mselectAction() {

	// check for model parameter 'm' and load the associated model if it exists
	if ($this->getRequest()->getParam('m') &&
	$this->erfurtStore->getModel($this->getRequest()->getParam('m')) ) {

		// instanciate model
		$model = $this->erfurtStore->getModel($this->getRequest()->getParam('m'));

		// set active model in session
		$this->session->activeModel = $model;
		$this->session->activeModelUri = $model->modelURI;

		// redirect to requirements list
		$this->_redirect('/resource/list/?r=' . urlencode(SWORE_REQUIREMENT));
		exit();
	}

	// get strings for gui output
	$this->_strings = Zend_Registry::get('strings');

	// set view title
	$this -> view -> placeholder('title') -> set($this -> _strings -> sw -> selectmodel -> title);

	// set additional search paths for the view script
	$softiwkipath = $this->erfurtApp->getPluginManager()->getPluginRoot('SoftWiki');
	$this -> view -> addScriptPath($softiwkipath . '/templates/');

	// get available models again
	$models = $this -> erfurtApp -> getStore() -> listModels();
	$sworeModels = array();

	//check every with a short sparql if it is a swore-model
	foreach ($models as $model) {
		$sparqlstring = 'ASK WHERE {<' . $model->getModelURI() .'> <' . EF_OWL_IMPORTS . '> <' . SWORE_MODEL . '>}';
		if ($model -> sparqlQuery($sparqlstring)) {
			// query for more data used by view
			$sparqlstring = 'SELECT * WHERE { <' . $model->getModelURI() . '> <' . EF_RDFS_LABEL . '> ?label' .
				' . OPTIONAL { <' . $model->getModelURI() . '> <' . EF_RDFS_COMMENT . '> ?comment . } }';
			$res = $model -> sparqlQuery($sparqlstring);

			$temp = array(
					'label' => $res[0]['label']->getLabel() ,
					'uri' => $model -> getModelURI());

			if (is_object($res[0]['comment'])) {
				$temp['comment'] = $res[0]['comment']->getLabel();
			} else {
				$temp['comment'] = '';
			}

			//Last 5 Requirements

			$prefix= 'PREFIX req: <http://ns.softwiki.de/req/>' 
					.' PREFIX dc: <http://purl.org/dc/elements/1.1/>'
					.' PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>'
					.' PREFIX tags: <http://www.holygoat.co.uk/owl/redwood/0.1/tags/> ';
			$sparqlstring='SELECT DISTINCT ?u ?t ?m ?s WHERE { 	?u rdf:type req:Requirement.  ?u dc:title ?t. ?u dcterms:modified ?m. ?u req:isLastModifiedBy ?s }'
					.' ORDER BY DESC(?m) LIMIT 5';

			$res = $model -> sparqlQuery($prefix.' '.$sparqlstring);
			$requirements='<ul>';
			for ($i=0;$i<5;$i++) {
				if (is_object($res[$i]['t'])) { 
					$requirements .= '<li><a href="open/?requirement='.$res[$i]['u']->getLabel().'&m='.$temp['uri'].'" >'.$res[$i]['t']->getLabel().'</a> ('. Erfurt_Util::dateDifference($res[$i]['m']->getLabel(), false, 3).' von '.ltrim(strrchr($res[$i]['s']->getLabel(),'/'),'/').') </li>'; 
				}
			}
			$requirements .= '</ul>';
			$temp['requirements']=$requirements;

			// Count_Requirements
			$sparqlstring='SELECT DISTINCT ?u WHERE { ?u rdf:type req:Requirement }';
			$res = $model -> sparqlQuery($prefix.' '.$sparqlstring);
			$temp['count_req']=count($res);	


			$sworeModels[] = $temp;
		}
	}

	// assign view variables
	$this -> view -> models = $sworeModels;
	
	
	
	
	
}

/**
 * Enter description here...
 *
 * @param unknown_type $uri
 * @return unknown
 */
private function _getReqId($uri) {
	$matches = array();
	preg_match("{[0-9]+$}", $uri, $matches);

	return $matches[0];
}

/**
 * Service for file-forms
 *
 */
public function fileAction() {
	// as a service it needs no layout template
	$this->_helper->layout->disableLayout();
	// add path for correct template
	$this->view->addScriptPath(REAL_BASE.'plugins/SoftWiki/templates/');

	// needed variables for OntoWiki file management
	$sysontModel = Zend_Registry::get('sysontModel');
	$fMime = $this -> config -> filemanager -> file -> mimeproperty;
	$fClass = $this -> config -> filemanager -> file -> ofclass;
	$fModel = $this -> config -> filemanager -> file -> ofmodel;

	$resourceURI = $this -> session -> activeResourceUri;

	if ($type = $this->_request->getParam('type')) {
		
		if ($type == 'exist') {
			// new files and temp array
			$files = array();
			$temp = array();
			// search files that aren't linked yet
			$query = 'SELECT * WHERE { <' . $resourceURI . '> <' . SWORE_FILE_PROPERTY . '> ?file' .
            	' . }';
			foreach ( $this -> model -> sparqlQuery($query) as $qr) {
				$temp[] = $qr['file']->getURI();
			}
			$query = 'SELECT DISTINCT ?mime ?uri ?label WHERE {?uri <' . $fModel . '> <' .
			$this -> model -> modelURI . '> . ' . '?uri <' . $fMime . '> ?mime . ?uri a <' .
			$fClass . '> . }';
			foreach($sysontModel -> sparqlQuery($query) as $qrkey => $qr) {
				if (in_array($qr['uri']->getURI(),$temp)) {
					// do nothing
				} else {
					// Add file with label to filearray later assigned to view
					$files[] = array(
							'uri' => $qr['uri']->getURI(),
							'label' => $this -> model -> resourceF ($qr['uri']->getURI())
					-> getTitle()
					);
				}
			}
			$this -> view -> files = $files;

		} elseif ($type == 'new') {
			$this -> view -> useService = 1;

			// Check if upload is possible
			if (is_writable(REAL_BASE . $this -> config -> filemanager -> path)) {
				// Generate default URI for file and set it in view
				$size = sizeof(scandir(REAL_BASE . $this -> config ->filemanager -> path));
				$name = 'file_' . ($size - 2 );
				$defaultUri .= str_replace($this -> config -> erfurtUriBase, '', $this -> config -> erfurtUrlBase);
				$defaultUri .= $this -> config -> uriBase;
				$defaultUri .= $this -> config -> filemanager -> path . $name;
				$this -> view -> defaultUri = $defaultUri;
			} else {
				echo '<div class="window"><h1 class="title">Upload Error</h1>' . PHP_EOL .
					'<div class="content">' . PHP_EOL .
					'<div class="messagebox error">directory ' . 
				REAL_BASE . $this -> config -> filemanager -> path .
					' not writeable</div>' . PHP_EOL .
					'</div>' . PHP_EOL .
					'</div>' . PHP_EOL ;
				exit();
			}
			//				$this -> view -> resourceURI = $resourceURI;
		} elseif ($type == 'add' && $this->_request->getParam('resources') ) {
		
			$uris = $this ->_request->getParam('resources');
			
			if (get_magic_quotes_gpc()) {
				$uris = json_decode(stripslashes($uris));
			} else {
				$uris = json_decode($uris);
			}

			foreach ($uris as $uri) {
					
				$label = $this->model->resourceF($uri)->getTitle();

				if ($this -> model -> add ($resourceURI,SWORE_FILE_PROPERTY,$uri)) {
					$msg_success[] = $this -> strings -> sw -> files -> linking .
						 ': ' . $label . ' ' . $this -> strings -> sw -> common -> success;
				} else {
					$msg_error[] = $this -> strings -> sw -> files -> linking .
						 ': ' . $label . ' ' . $this -> strings -> sw -> common -> failed;
				}
			}
			$this -> view -> messages = array();
			$this -> view -> messages['error'] = $msg_error;
			$this -> view -> messages['success'] = $msg_success;
		} elseif ($type == 'unlink' && ( $uris = $this ->_request->getParam('resources') )) {

			if (get_magic_quotes_gpc()) {
				$uris = json_decode(stripslashes($uris));
			} else {
				$uris = json_decode($uris);
			}

			foreach ($uris as $uri) {
				$label = $this->model->resourceF($uri)->getTitle();
				if ($this -> model -> remove ($resourceURI,SWORE_FILE_PROPERTY,$uri)) {
					$msg_success[] = $this -> strings -> sw -> files -> unlinking .
						 ': ' . $label . ' ' . $this -> strings -> sw -> common -> success;
				} else {
					$msg_error[] = $this -> strings -> sw -> files -> unlinking .
						 ': ' . $label . ' ' . $this -> strings -> sw -> common -> failed;
				}
			}
			$this -> view -> messages = array();
			$this -> view -> messages['error'] = $msg_error;
			$this -> view -> messages['success'] = $msg_success;

		} else {
			// do nothing
		}

		// set template to softwiki/file_$type.php
		$this->_helper->viewRenderer->renderScript('softwiki/file_' . $type . '.php');
			
	} else {
		//do nothing
	}
}

/**
 * Service for decison-forms
 *
 */
public function decAction() {
	// as a service it needs no layout template
	$this->_helper->layout->disableLayout();
	// add path for correct template
	$this->view->addScriptPath(REAL_BASE.'plugins/SoftWiki/templates/');

	// needed variables for OntoWiki decision management
	$sysontModel = Zend_Registry::get('sysontModel');
	$fMime = $this -> config -> filemanager -> file -> mimeproperty;
	$fClass = $this -> config -> filemanager -> file -> ofclass;
	$fModel = $this -> config -> filemanager -> file -> ofmodel;

	$resourceURI = $this -> session -> activeResourceUri;

	if ($type = $this->_request->getParam('type')) {
		
		if ($type == 'exist') {
			// new files and temp array
			$decisions = array();
			$temp = array();
			$temp1 = array();

			
			$query = 'SELECT DISTINCT * WHERE {?sol <' . EF_RDFS_LABEL . '> ?label. ?sol <http://purl.org/dc/elements/1.1/description> ?description. ?sol <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> ?t . ?t <' . EF_RDFS_LABEL . '> ?type. } ORDER BY ASC(?sol)';

			$qr = $this -> model -> sparqlQuery($query);
			if (is_array($qr)) {
				foreach ($qr as $f) {

					$sql =  "SELECT count(object) as Anzahl FROM statements WHERE ";
					$sql .= "predicate='http://ns.softwiki.de/req/solution' ";
					$sql .= "and subject!='http://ns.softwiki.de/req/Requirement' ";
					$sql .= "and object='".$f['sol']->getURI()."' ";
					$sql .= "and modelID IN (" . $this->model->getModelIds() . ") ";

					$sqlResult = $this->model->dbConn->Execute($sql);
					$DecCount=0;
					
					while ($row = $sqlResult->FetchRow()) {
						$DecCount=(int)$row[0];	
					};				

					$DecFavorite='';

					$sql =  "SELECT count(object) as Anzahl FROM statements WHERE ";
					$sql .= "predicate='http://ns.softwiki.de/req/solutionFavorite' ";
					$sql .= "and subject!='http://ns.softwiki.de/req/Requirement' ";
					$sql .= "and object='".$f['sol']->getURI()."' ";
					$sql .= "and modelID IN (" . $this->model->getModelIds() . ") ";

					$sqlResult = $this->model->dbConn->Execute($sql);
					
					while ($row = $sqlResult->FetchRow()) {
						if ((int)$row[0]>0) { $DecCount=$DecCount+(int)$row[0]; $DecFavorite='top'; }
					};				

					$sql =  "SELECT count(object) as Anzahl FROM statements WHERE ";
					$sql .= "predicate='http://ns.softwiki.de/req/solutionExcluded' ";
					$sql .= "and subject!='http://ns.softwiki.de/req/Requirement' ";
					$sql .= "and object='".$f['sol']->getURI()."' ";
					$sql .= "and modelID IN (" . $this->model->getModelIds() . ") ";

					$sqlResult = $this->model->dbConn->Execute($sql);
					
					while ($row = $sqlResult->FetchRow()) {
						if ((int)$row[0]>0) { $DecCount=0; $DecFavorite='down'; }
					};				



					$temp = array(
    					'uri' => $f['sol']->getURI(),
    					'label' => $f['label']->getLabel(),
   					'description' => $f['description']->getLabel(),
  					'type' => $f['type']->getLabel(),
  					'count' => $DecCount,
					'favorite' => $DecFavorite);
					$decisions[] = $temp;
				}
			}
			$this -> view -> decisions = $decisions;



		} elseif ($type == 'add' && $this->_request->getParam('resources') ) {
		
			$uris = $this ->_request->getParam('resources');
			
			if (get_magic_quotes_gpc()) {
				$uris = json_decode(stripslashes($uris));
			} else {
				$uris = json_decode($uris);
			}

			foreach ($uris as $uri) {
					
				$label = $this->model->resourceF($uri)->getTitle();

				if ($this -> model -> add ($resourceURI,SWORE_FILE_PROPERTY,$uri)) {
					$msg_success[] = $this -> strings -> sw -> files -> linking .
						 ': ' . $label . ' ' . $this -> strings -> sw -> common -> success;
				} else {
					$msg_error[] = $this -> strings -> sw -> files -> linking .
						 ': ' . $label . ' ' . $this -> strings -> sw -> common -> failed;
				}
			}
			$this -> view -> messages = array();
			$this -> view -> messages['error'] = $msg_error;
			$this -> view -> messages['success'] = $msg_success;
		} elseif ($type == 'unlink' && ( $uris = $this ->_request->getParam('resources') )) {

			if (get_magic_quotes_gpc()) {
				$uris = json_decode(stripslashes($uris));
			} else {
				$uris = json_decode($uris);
			}

			foreach ($uris as $uri) {
				$label = $this->model->resourceF($uri)->getTitle();
				if ($this -> model -> remove ($resourceURI,SWORE_SOLUTION_PROPERTY,$uri)) {
					$this -> model -> remove ($resourceURI,SWORE_FAVSOLUTION_PROPERTY,$uri);
					$this -> model -> remove ($resourceURI,SWORE_EXSOLUTION_PROPERTY,$uri);
					$msg_success[] = $this -> strings -> sw -> decisions -> unlinking .
						 ': ' . $label . ' ' . $this -> strings -> sw -> common -> success;
				} else {
					$msg_error[] = $this -> strings -> sw -> decisions -> unlinking .
						 ': ' . $label . ' ' . $this -> strings -> sw -> common -> failed;
				}
			}
			$this -> view -> messages = array();
			$this -> view -> messages['error'] = $msg_error;
			$this -> view -> messages['success'] = $msg_success;

		} elseif ($type == 'top' && ( $uris = $this ->_request->getParam('resources') )) {

			if (get_magic_quotes_gpc()) {
				$uris = json_decode(stripslashes($uris));
			} else {
				$uris = json_decode($uris);
			}

			foreach ($uris as $uri) {
				$label = $this->model->resourceF($uri)->getTitle();
				if ($this -> model -> add ($resourceURI,SWORE_FAVSOLUTION_PROPERTY,$uri)) {
					$this -> model -> remove ($resourceURI,SWORE_EXSOLUTION_PROPERTY,$uri);
					$msg_success[] = $this -> strings -> sw -> decisions -> toplinking .
						 ': ' . $label . ' ' . $this -> strings -> sw -> common -> success;
				} else {
					$msg_error[] = $this -> strings -> sw -> decisions -> toplinking .
						 ': ' . $label . ' ' . $this -> strings -> sw -> common -> failed;
				}
			}
			$this -> view -> messages = array();
			$this -> view -> messages['error'] = $msg_error;
			$this -> view -> messages['success'] = $msg_success;

		} elseif ($type == 'down' && ( $uris = $this ->_request->getParam('resources') )) {

			if (get_magic_quotes_gpc()) {
				$uris = json_decode(stripslashes($uris));
			} else {
				$uris = json_decode($uris);
			}

			foreach ($uris as $uri) {
				$label = $this->model->resourceF($uri)->getTitle();
				if ($this -> model -> add ($resourceURI,SWORE_EXSOLUTION_PROPERTY,$uri)) {
					$this -> model -> remove ($resourceURI,SWORE_FAVSOLUTION_PROPERTY,$uri);
					$msg_success[] = $this -> strings -> sw -> decisions -> downlinking .
						 ': ' . $label . ' ' . $this -> strings -> sw -> common -> success;
				} else {
					$msg_error[] = $this -> strings -> sw -> decisions -> downlinking .
						 ': ' . $label . ' ' . $this -> strings -> sw -> common -> failed;
				}
			}
			$this -> view -> messages = array();
			$this -> view -> messages['error'] = $msg_error;
			$this -> view -> messages['success'] = $msg_success;

		} elseif ($type == 'untop' && ( $uris = $this ->_request->getParam('resources') )) {

			if (get_magic_quotes_gpc()) {
				$uris = json_decode(stripslashes($uris));
			} else {
				$uris = json_decode($uris);
			}

			foreach ($uris as $uri) {
				$label = $this->model->resourceF($uri)->getTitle();
				if ($this -> model -> remove ($resourceURI,SWORE_FAVSOLUTION_PROPERTY,$uri)) {
					$msg_success[] = $this -> strings -> sw -> decisions -> untoplinking .
						 ': ' . $label . ' ' . $this -> strings -> sw -> common -> success;
				} else {
					$msg_error[] = $this -> strings -> sw -> decisions -> untoplinking .
						 ': ' . $label . ' ' . $this -> strings -> sw -> common -> failed;
				}
			}
			$this -> view -> messages = array();
			$this -> view -> messages['error'] = $msg_error;
			$this -> view -> messages['success'] = $msg_success;

		} elseif ($type == 'undown' && ( $uris = $this ->_request->getParam('resources') )) {

			if (get_magic_quotes_gpc()) {
				$uris = json_decode(stripslashes($uris));
			} else {
				$uris = json_decode($uris);
			}

			foreach ($uris as $uri) {
				$label = $this->model->resourceF($uri)->getTitle();
				if ($this -> model -> remove ($resourceURI,SWORE_EXSOLUTION_PROPERTY,$uri)) {
					$msg_success[] = $this -> strings -> sw -> decisions -> undownlinking .
						 ': ' . $label . ' ' . $this -> strings -> sw -> common -> success;
				} else {
					$msg_error[] = $this -> strings -> sw -> decisions -> undownlinking .
						 ': ' . $label . ' ' . $this -> strings -> sw -> common -> failed;
				}
			}
			$this -> view -> messages = array();
			$this -> view -> messages['error'] = $msg_error;
			$this -> view -> messages['success'] = $msg_success;

		} elseif ($type == 'link' && ( $uris = $this ->_request->getParam('resources') )) {

			if (get_magic_quotes_gpc()) {
				$uris = json_decode(stripslashes($uris));
			} else {
				$uris = json_decode($uris);
			}

			foreach ($uris as $uri) {
				$label = $this->model->resourceF($uri)->getTitle();
				if ($this -> model -> add ($resourceURI,SWORE_SOLUTION_PROPERTY,$uri)) {
					$msg_success[] = $this -> strings -> sw -> decisions -> linking .
						 ': ' . $label . ' ' . $this -> strings -> sw -> common -> success;
				} else {
					$msg_error[] = $this -> strings -> sw -> decisions -> linking .
						 ': ' . $label . ' ' . $this -> strings -> sw -> common -> failed;
				}
			}
			$this -> view -> messages = array();
			$this -> view -> messages['error'] = $msg_error;
			$this -> view -> messages['success'] = $msg_success;

		} else {
			// do nothing
		}

		// set template to softwiki/dec_$type.php
		$this->_helper->viewRenderer->renderScript('softwiki/dec_' . $type . '.php');
			
	} else {
		//do nothing
	}
}


/**
 *
 */
public function extratabAction() {

	// no layout- and template-engine needed here
	$this->_helper->viewRenderer->setNoRender();
	$this->_helper->layout->disableLayout();

	$params = $_GET;
	$render = array();
	if ($params['tab']) {
		foreach ($params['tab'] as $tab) {
			if (! strstr($tab['string'],';')) {
				eval('$string = $this->strings->' . str_replace('.','->',$tab['string']) . ';');
				eval('$tooltipstr = $this->strings->tooltip->' . str_replace('.','->',$tab['string']) . ';');
				$link = $tab['link'];
				$render[$string] = $link;
				$tooltip[$string] =  $tooltipstr;
			} else {
				// somebody tried to inject php code!
				exit();
			}
		}
	}

	echo '<ul class="tabs">' . PHP_EOL;
	$i = 0;

	foreach ($render as $string => $link) {

		if ($i == $params['active']) {
			$active = 'active';
		} else {
			$active = '';
		}

		echo '  <li class="' . $active . '">' . PHP_EOL .
        		'    <a href="' . $link . '" title="' . $tooltip[$string] . '">' . $string . '</a>' . PHP_EOL .
        		'  </li>' . PHP_EOL;
		$i++;
	}

	echo '</ul>' . PHP_EOL;

}

/**
 *  Service to handle backend operations in softwiki relation view window
 */
public function relationsserviceAction() {

	// no layout- and template-engine needed here
	$this->_helper->viewRenderer->setNoRender();
	$this->_helper->layout->disableLayout();


	$temp = array();
	$params = $_GET;
	$session = Zend_Registry::get('session');
	$lang = $session->modelLanguage;
	$subjectURI = urldecode($params['r']);
	$objectURI = urldecode($params['object']);
	preg_match("{[0-9]+$}", $subjectURI, $temp);
	$subjectSwid = $temp[0];
	preg_match("{[0-9]+$}", $objectURI, $temp);
	$objectSwid = $temp[0];
	
	$defined = $this->model->sparqlQuery('SELECT * WHERE {<' . $subjectURI . '> ?p <' 
	         . $objectURI . '> . OPTIONAL {<' . $objectURI . '> ?ip <' . $subjectURI . '> . } }');
	
	// Output of information window if no update requested, doing data operations (del or add) else
	if (!isset($params['reqRelation'])) {
	    $query = 'SELECT * WHERE {?x a <' . EF_OWL_OBJECT_PROPERTY . '> . '
	           . '?x <' . EF_RDFS_DOMAIN . '> <' . SWORE_REQUIREMENT . '> . '
	           . '?x <' . EF_RDFS_RANGE . '> <' . SWORE_REQUIREMENT . '> .}';
	    $allRelations = $this->model->sparqlQuery($query);
	
	   
	    $strReqA =  '<br/><strong>' . urldecode($params['r_title']) . ' (#' . $subjectSwid .')'
	    	     .  '</strong><br/>';
	    	 
	    $strReqB =  '<br/><strong>' . urldecode($params['object_title']) . ' (#' . $objectSwid .')'
	    	     .  '</strong><br/>';

	    $content .= '<p style="margin-left:auto; margin-right:auto;">'
	             . sprintf($this->strings->sw->reqrelation->between,$strReqA,$strReqB)
	             . '</p>';
	    	     
	    $content .= '<form id="input-reqRelation" action="#">'
	             .  '  <input type="hidden" name="r" value="' . $subjectURI . '"/>'
	             .  '  <input type="hidden" name="object" value="' . $objectURI . '"/><br/>';
	    $checked = 'checked="checked"';
	    foreach ($allRelations as $relation) {
	    	if (!empty($defined) && $defined[0]['p']->getURI() == $relation['x']->getURI() ) {
	    	    $content .= '  <p><input type="radio" name="reqRelation" value="' . $relation['x']->getURI() 
		                 .  '" checked="checked"> ' . $relation['x']->getTitle($lang) . '</p>';
		        $checked = '';
	    	} else {
	    	    $content .= '  <p><input type="radio" name="reqRelation" value="' . $relation['x']->getURI() 
		                 .  '"> ' . $relation['x']->getTitle($lang) . '</p>';
	    	}

	    }
	    $content .= '  <br/><p><input type="radio" name="reqRelation" value="null" ' . $checked . '> '
	       . $this->strings->sw->reqview->deletebutton . '</p>';


	    $content .= '</form>';
	    $content .= '<div>'
	             .  '  <a class="float-left input-reqRelation-cancel minibutton" title="'
	             .  $this->strings->sw->reqview->cancelbutton . '">'
	             .  '    <img style="vertical-align:middle; margin:0.2em;" alt=""'
	             .  ' src="../../themes/default/images/icon-delete.png"/>'
	             .  $this->strings->sw->reqview->cancelbutton
	             .  '  </a>'
	             .  '  <a class="float-left input-reqRelation-ok minibutton" title="'
	             .  $this->strings->sw->reqview->submitchangesbutton . '">'
	             .  '    <img style="vertical-align:middle; margin:0.2em;" alt=""'
	             .  ' src="../../themes/default/images/icon-edit.png"/>'
	             .  $this->strings->sw->reqview->submitchangesbutton
	             .  '  </a>'
	             .  '</div>';
	    $this->view->content = $content;
	    $this->view->title = $this->strings->sw->reqview->reqeditbutton;
	    echo $this->view->render('micro/window.php');
	} else {

	    $predicateURI = $params['reqRelation'];
	    $subject = $this->model->resourceF($subjectURI);
	    $object = $this->model->resourceF($objectURI);
	    
	    if ($predicateURI != "null") {
	    	$predicate = $this->model->resourceF($predicateURI);
	    	$invpredicate = $predicate->getPropertyValue(EF_OWL_NS . 'inverseOf');
	    } else {
	    	$predicate = null;
	    	$invpredicate = null;
	    }
	    
		// Delete existing relations
	    if (!empty($defined)) {
	        $existingProperty = $this->model->resourceF($defined[0]['p']->getURI());
	        if ( empty($defined[0]['ip']) ) {
	            $subject->setPropertyValue($defined[0]['p'],array());
	            echo 'state:OK; ' . PHP_EOL . 'stateDetails: del:'
	               . $defined[0]['p']->getURI() . ';' . PHP_EOL;
	        } else {
	        	$subject->setPropertyValue($defined[0]['p'],array());
	        	$object->setPropertyValue($defined[0]['ip'],array());
	            echo 'state:OK; ' . PHP_EOL . 'stateDetails: del:' . $defined[0]['p']->getURI()
	               . '; del:' . $defined[0]['ip']->getURI() . ';' . PHP_EOL;
	        }
	    }
	    
	    // Create new relations (if needed)
	    if  (!is_null($predicate)) {
	    	if (is_null($invpredicate)) {
	    		$subject->setPropertyValue($predicate,$object);
	    		echo 'state:OK; ' . PHP_EOL . 'stateDetails: add:'
	    		   . $predicate->getURI() . ';' . PHP_EOL;
	    	} else {
	    	    $subject->setPropertyValue($predicate,$object);
	    	    $object->setPropertyValue($invpredicate,$subject);
	            echo 'state:OK; ' . PHP_EOL . 'stateDetails: add:' . $predicate->getURI()
	               . '; add:' . $invpredicate->getURI() . ';' . PHP_EOL;
	    	}
	    }
	    
	    // no operations performed
	    if (empty($defined) && is_null($predicate)){
	    	echo 'state:warning; warning:no operations performed;';
	    }
	}
	

}
    /**
     *
     */
    public function openAction()
    {   

        // redirect if no access.    
        if (!$this->model) {
            $this->_redirect('');
        }

        // overwrite default sidebar
		$this->view->addScriptPath(dirname(__FILE__));

		$this->view->placeholder('sidebar')->set($this->view->partial('sidebar.php'));

        $this->view->addScriptPath(REAL_BASE . 'plugins/SoftWiki/templates/');
        $this->view->resourceUri = $_GET['requirement'];

    }




    public function linkprojectAction()
    {   

	$store1 = Zend_Registry::get('erfurt')->getStore();

        // redirect if no access.    
        if (!$this->model) {
            $this->_redirect('');
        }

        // overwrite default sidebar
		$this->view->addScriptPath(dirname(__FILE__));

		$this->view->placeholder('sidebar')->set($this->view->partial('sidebar.php'));

        $this->view->addScriptPath(REAL_BASE . 'plugins/SoftWiki/templates/');
        $this->view->resourceUri = $_GET['r'];


		// get current Project
		$doapProjectSQL = "SELECT DISTINCT subject FROM statements WHERE predicate = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#type' AND object = 'http://usefulinc.com/ns/doap#Project' AND modelID = " . $this->model->getModelID();
		$doapProjectURI="";


		$resultData = $store1->sqlQuery($doapProjectSQL);
		foreach ($resultData as $doapProject) {
			$doapProjectURI = $doapProject[0];
			break;			
		};
		if ($doapProjectURI!="") {
			

			// add current Project
			$newStatement=new Statement(new Resource($this->view->resourceUri),new Resource("http://ns.softwiki.de/req/isRequirementOf"), new Resource($doapProjectURI));
			$this->model->addStatementArray(array($newStatement));

		};

    }


    public function unlinkprojectAction()
    {   

	$store1 = Zend_Registry::get('erfurt')->getStore();

        // redirect if no access.    
        if (!$this->model) {
            $this->_redirect('');
        }

        // overwrite default sidebar
		$this->view->addScriptPath(dirname(__FILE__));

		$this->view->placeholder('sidebar')->set($this->view->partial('sidebar.php'));

        $this->view->addScriptPath(REAL_BASE . 'plugins/SoftWiki/templates/');
        $this->view->resourceUri = $_GET['r'];


		// get current Project
		$doapProjectSQL = "SELECT DISTINCT subject FROM statements WHERE predicate = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#type' AND object = 'http://usefulinc.com/ns/doap#Project' AND modelID = " . $this->model->getModelID();
		$doapProjectURI="";


		$resultData = $store1->sqlQuery($doapProjectSQL);
		foreach ($resultData as $doapProject) {
			$doapProjectURI = $doapProject[0];
			break;			
		};
		if ($doapProjectURI!="") {
			
			$this->model->remove($this->view->resourceUri, SWORE_ISREQUIREMENTOF ,$doapProjectURI);
		
		};





    }


    public function adminAction()
    {   


	// get strings for gui output
	$this->_strings = Zend_Registry::get('strings');

	// set view title
	$this -> view -> placeholder('title') -> set($this -> _strings -> sw -> admin -> title);

	// set additional search paths for the view script
	$softiwkipath = $this->erfurtApp->getPluginManager()->getPluginRoot('SoftWiki');
	$this -> view -> addScriptPath($softiwkipath . '/templates/');


	$admintext ="";
    $this->_ac = Zend_Registry::get('ac');

	$isOntoWikiAllowed = ($this->_ac->isActionAllowed('SoftWikiUseOntoWiki')) ? true : false;

	if (!$isOntoWikiAllowed) {
		$admintext .= 'Sie sind haben keine Berechtigungen diese SoftWiki-Installation zu administrieren.'; 
	// assign view variables
    } else {

		// check for model parameter 'aa'=admin action
		if ($this->getRequest()->getParam('aa') ) {

			// some Admin-Action take place
			$aa= $this->getRequest()->getParam('aa');
			$validate= $this->getRequest()->getParam('validate');
			$uri= $this->getRequest()->getParam('uri');
			$label= $this->getRequest()->getParam('label');
			$comment= $this->getRequest()->getParam('comment');
			$template= $this->getRequest()->getParam('template');

			// get store and model id
			$store = Zend_Registry::get('erfurt')->getStore();


			// add Vocabularies
			if (($aa=="addvocabularies")&&($validate=="create")) {

				//FOAF
				$filehandler = fopen("plugins/SoftWiki/models/imports/foaf.rdf","r");	
				$data = '';
				while (!feof($filehandler)) $data .= fgets($filehandler,1000);
				fclose($filehandler);
				$uri="http://xmlns.com/foaf/0.1/";
				if($store->modelExists($uri)===false) {
					$model = $store->getNewModel($uri, $uri);
					$memmodel = new MemModel();
					try{
						$memmodel->loadFromString($data, "rdf");
						$model->addModel($memmodel);
					}
					catch(Exception $e){
						echo get_class($e) . ': ' . $e->getMessage();
					}
				}
				//SWORE
				$filehandler = fopen("plugins/SoftWiki/models/imports/swore.rdf","r");	
				$data = '';
				while (!feof($filehandler)) $data .= fgets($filehandler,1000);
				fclose($filehandler);
				$uri="http://ns.softwiki.de/req/";
				if($store->modelExists($uri)===false) {
					$model = $store->getNewModel($uri, $uri);
					$memmodel = new MemModel();
					try{
						$memmodel->loadFromString($data, "rdf");
						$model->addModel($memmodel);
					}
					catch(Exception $e){
						echo get_class($e) . ': ' . $e->getMessage();
					}
				}
				//TAGS
				$filehandler = fopen("plugins/SoftWiki/models/imports/tags.rdf","r");	
				$data = '';
				while (!feof($filehandler)) $data .= fgets($filehandler,1000);
				fclose($filehandler);
				$uri="http://www.holygoat.co.uk/owl/redwood/0.1/tags/";
				if($store->modelExists($uri)===false) {
					$model = $store->getNewModel($uri, $uri);
					$memmodel = new MemModel();
					try{
						$memmodel->loadFromString($data, "rdf");
						$model->addModel($memmodel);
					}
					catch(Exception $e){
						echo get_class($e) . ': ' . $e->getMessage();
					}
				}
				//SKOS
				$filehandler = fopen("plugins/SoftWiki/models/imports/skos.rdf","r");	
				$data = '';
				while (!feof($filehandler)) $data .= fgets($filehandler,1000);
				fclose($filehandler);
				$uri="http://www.w3.org/2004/02/skos/core";
				if($store->modelExists($uri)===false) {
					$model = $store->getNewModel($uri, $uri);
					$memmodel = new MemModel();
					try{
						$memmodel->loadFromString($data, "rdf");
						$model->addModel($memmodel);
					}
					catch(Exception $e){
						echo get_class($e) . ': ' . $e->getMessage();
					}
				}

				// redirect to Admin
				$this->_redirect('/softwiki/admin');
				exit();
			};
			// create a new Project
			if (($aa=="newproject")&&($validate=="create")) {


				//Sample Solutions
				$filehandler = fopen("plugins/SoftWiki/models/project/solutions_basic.rdf","r");	
				$data = '';
				while (!feof($filehandler)) $data .= fgets($filehandler,1000);
				fclose($filehandler);

				//find URI
				$uri_1=0;$uri_2=0;
				$uri_1=strpos($data,'<!ENTITY base "',0)+15;
				$uri_2=strpos($data,'">',$uri_1);
				$uri_i=substr($data,$uri_1,$uri_2-$uri_1);

				if($store->modelExists($uri_i)===false) {
					$model = $store->getNewModel($uri_i, $uri_i);
					$memmodel = new MemModel();
					try{
						$memmodel->loadFromString($data, "rdf");
						$model->addModel($memmodel);
					}
					catch(Exception $e){
						echo get_class($e) . ': ' . $e->getMessage();
					}
				}


				//Sample Topics
				$filehandler = fopen("plugins/SoftWiki/models/project/topics_ntier.rdf","r");	
				$data = '';
				while (!feof($filehandler)) $data .= fgets($filehandler,1000);
				fclose($filehandler);

				//find URI
				$uri_1=0;$uri_2=0;
				$uri_1=strpos($data,'<!ENTITY base "',0)+15;
				$uri_2=strpos($data,'">',$uri_1);
				$uri_i=substr($data,$uri_1,$uri_2-$uri_1);

				if($store->modelExists($uri_i)===false) {
					$model = $store->getNewModel($uri_i, $uri_i);
					$memmodel = new MemModel();
					try{
						$memmodel->loadFromString($data, "rdf");
						$model->addModel($memmodel);
					}
					catch(Exception $e){
						echo get_class($e) . ': ' . $e->getMessage();
					}
				}


				//load the SW-Project Template
				// read template file to data string
				$data='';
				$filehandler = fopen("plugins/SoftWiki/models/project/softwiki_project.rdf","r");	
				$data = '';
				while (!feof($filehandler)) {
					$data .= fgets($filehandler,1000);
				}
				fclose($filehandler);

				$data = ereg_replace('rdfs:label="Project Title"', 'rdfs:label="'.$label.'"',  $data);

				$data = ereg_replace('<!ENTITY base "http://softwiki.de/models/prototype/project/">', '<!ENTITY base "'.$uri.'">',  $data);
				$data = ereg_replace('dc:description="Project Description"', 'dc:description="'.$comment.'"',  $data);
				$data = ereg_replace('<rdfs:comment>Project Description.</rdfs:comment>', '<rdfs:comment>'.$comment.'</rdfs:comment>',  $data);

//				echo "Hallo";exit;				

				$model = $store->getNewModel($uri, $uri);
				$memmodel = new MemModel();
				try{
					$memmodel->loadFromString($data, "rdf");
					$model->addModel($memmodel);
				}
				catch(Exception $e){
					echo get_class($e) . ': ' . $e->getMessage();
				}


				// redirect to Requirements List
				$this->_redirect('/resource/list/?r=' . urlencode('req:Requirement').'&m=' . urlencode($uri));
				exit();


			};
		} else {
			// AdminAction is not set
			$aa='';
		}

		$admintext .= '<fieldset><legend>SoftWiki Administration</legend>';

		// Admin add Vocabularies
		$admintext .= '<fieldset class="form" id="admin1"><legend>Add a all related Vokabularies</legend>'
				. '<form method="post" action="admin" enctype="multipart/form-data" class="row-input input-justify-left">'
        		. '<input type="hidden" class="text" name="aa" value="addvocabularies" />'
        		. '<input type="hidden" class="text" name="validate" value="create" />'
   	 			. '<div class="actionbuttons">'
        		. '<button type="submit">Add</button>'
    			. '</div>'
				. '</form>'
    			. '</fieldset>';
		
		 

		// Admin add New Project
		$admintext .= '<fieldset class="form" id="admin2"><legend>Add a new SoftWiki-Project</legend>';
		if ($aa=='newproject') {
			$admintext .= ''
				. '<form method="post" action="admin" enctype="multipart/form-data" class="row-input input-justify-left">'
        		. '<input type="hidden" class="text" name="aa" value="newproject" />'
        		. '<input type="hidden" class="text" name="validate" value="create" />'
    			. '<div>'
        		. ' <label for="model-input">Project Model URI</label> '
        		. $this->getRequest()->getParam('uri')
        		. '<input type="hidden" class="text" name="uri" value="'.$this->getRequest()->getParam('uri').'" />'
        		. '</div>'
        		. '<div>'
        		. ' <label for="title-input">Project Title</label> '
        		. $this->getRequest()->getParam('label')
        		. '<input type="hidden" class="text" name="label" value="'.$this->getRequest()->getParam('label').'" />'
    			. '</div>'
        		. '<div>'
            	. '<label for="template-select">Project Template</label>'
        		. $this->getRequest()->getParam('template')
        		. '<input type="hidden" class="text" name="template" value="'.$this->getRequest()->getParam('template').'" />'
  			    . '</div>'
    			. '<div>'
	            . '<label for="code-input">Description</label> ' 
        		. $this->getRequest()->getParam('comment')
        		. '<input type="hidden" class="text" name="comment" value="'.$this->getRequest()->getParam('comment').'" />'
        		. '</div>'
   	 			. '<div class="actionbuttons">'
        		. '<button type="submit">Create, really.</button>'
    			. '</div>'
				. '</form>'
				. '</fieldset>';
	
		} else {
			$admintext .= ''
				. '<form method="post" action="admin" enctype="multipart/form-data" class="row-input input-justify-left">'
        		. '<input type="hidden" class="text" name="aa" value="newproject" />'
        		. '<input type="hidden" class="text" name="validate" value="first" />'
    			. '<div>'
        		. ' <label for="uri-input">Project Model URI</label> <br/>'
        		. '<input type="text" class="text" id="uri-input" name="uri" value="" /> e.g. http://project.softwiki.de/MyProject/'
        		. '</div>'
        		. '<div>'
        		. ' <label for="label-input">Project Title</label> <br/>'
        		. '<input type="text" class="text" id="label-input" name="label" value="" /> e.g. My Project'
    			. '</div>'
        		. '<div>'

/*
            	. '<label for="template-select">Project Template</label>'
           	. '<span id="select-container-type-select" class="SelectNew">'
				. '<select name="template" class="SelectNew" id="template-select">';


			// create an array to hold directory list
			$modellist = array();

			// create a handler for the directory
			$dirhandler = opendir("plugins/SoftWiki/models/");

			// keep going until all files in directory have been read
			while ($file = readdir($dirhandler)) {

				// if $file isn't this directory or its parent, 
				// add it to the results array
				if ($file != '.' && $file != '..' && (strpos($file,'.rdf')!==false) && (strpos($file,'topic')!==false))  {
					$admintext .= '<option value="'.$file.'" selected="selected">'.$file.'</option>';
				}

			}

			// tidy up: close the handler
			closedir($dirhandler);

			$admintext .= '</select>'
				. '</span>'
 
          	. '<br class="clearall" />'
  			    . '</div>'
*/
    			. '<div>'
	            . '<label for="comment-input">Description</label> <br/>' 
        		. '<input type="text" class="text" id="comment-input" name="comment" value="" /> e.g. This project is about....'
        		. '</div>'
   	 			. '<div class="actionbuttons">'
        		. '<button type="submit">Add</button>'
    			. '</div>'
				. '</form>'
				. '</fieldset>';
		}
		
	}




	$admintext .= '</fieldset>';
	$this -> view -> admintext = $admintext;


	}


}


